"""
  Simple Wrapper on the Job class to handle EvnDisp Analysis
  for the final Prod3b pass
  https://forge.in2p3.fr/issues/27528
                        February 8th 2018 - J. Bregeon
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections
# DIRAC imports
from DIRAC.Interfaces.API.Job import Job
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient


class EvnDisp3FinalJob(Job):
  """ Job extension class for EvnDisp Analysis,
    takes care of running converter and evndisp.
  """

  def __init__(self, cpuTime=432000):
    """ Constructor

    Keyword arguments:
    cpuTime -- max cpu time allowed for the job
    """
    Job.__init__(self)
    self.setCPUTime(cpuTime)
    # defaults
    self.setName('Evndisplay_CalibReco')
    self.package = 'evndisplay'
    self.program_category = 'calibimgreco'
    self.version = 'prod3b_d20180201'
    self.configuration_id = -1
    self.output_data_level = 1
    self.prefix = 'CTA.prod3Sb'
    self.layout_list = '3HB9-FD 3HB9-FG 3HB9-FA 3HB9-ND 3HB9-NG 3HB9-NA \
                            3HB9-TS-BB-FD 3HB9-TS-BB-FG 3HB9-TS-BB-FA \
                            3HB9-TS-BB-ND 3HB9-TS-BB-NG 3HB9-TS-BB-NA'
    self.focal_file = 'CTA.prod3b.EffectiveFocalLength.dat'
    self.calibration_file = 'prod3b.Paranal-20171214.ped.root'
    self.reconstructionparameter = 'EVNDISP.prod3.reconstruction.runparameter.NN.noLL'
    self.NNcleaninginputcard = 'EVNDISP.NNcleaning.dat'
    self.basepath = '/vo.cta.in2p3.fr/MC/PROD3/'
    self.fcc = FileCatalogClient()
    self.metadata = collections.OrderedDict()
    self.filemetadata = {}
    self.catalogs = json.dumps(['DIRACFileCatalog', 'TSCatalog'])
    self.ts_task_id = 0

  def setEvnDispMD(self, simtelMD):
    """ Set evndisplay meta data

    Parameters:
    simtelMD -- metadata dictionary as for simtel
    """
    # # Set evndisp directory meta data
    self.metadata['array_layout'] = simtelMD['array_layout']
    self.metadata['site'] = simtelMD['site']
    self.metadata['particle'] = simtelMD['particle']
    try:
      phiP = simtelMD['phiP']['=']
    except BaseException:
      phiP = simtelMD['phiP']
    self.metadata['phiP'] = phiP
    try:
      thetaP = simtelMD['thetaP']['=']
    except BaseException:
      thetaP = simtelMD['thetaP']
    self.metadata['thetaP'] = thetaP
    self.metadata[self.program_category + '_prog'] = 'evndisp'
    self.metadata[self.program_category + '_prog_version'] = self.version
    self.metadata['data_level'] = self.output_data_level
    self.metadata['configuration_id'] = self.configuration_id

  def setupWorkflow(self, debug=False):
    """ Setup job workflow by defining the sequence of all executables
        All parameters shall have been defined before that method is called.
    """
    # step 1 -- debug
    iStep = 1
    if debug:
      lsStep = self.setExecutable('/bin/ls -alhtr',
                                  logFile='LS_Init_Log.txt')
      lsStep['Value']['name'] = 'Step%i_LS_Init' % iStep
      lsStep['Value']['descr_short'] = 'list files in working directory'
      iStep += 1

    # step 2
    swStep = self.setExecutable('cta-prod3-setupsw',
                                arguments='%s %s' % (self.package,
                                                     self.version),
                                logFile='SetupSoftware_Log.txt')
    swStep['Value']['name'] = 'Step%i_SetupSoftware' % iStep
    swStep['Value']['descr_short'] = 'Setup software'
    iStep += 1

    # step 2bis
    # arguments are nbFiles=0 (not used) and fileSize=100kB
    eivStep = self.setExecutable('cta-prod-verifysteps',
                                 arguments='analysisinputs 0 100',
                                 logFile='Verify_EvnDispInputs_Log.txt')
    eivStep['Value']['name'] = 'Step%i_VerifyEvnDispInputs' % iStep
    eivStep['Value']['descr_short'] = 'Verify EvnDisp Inputs'
    iStep += 1

    evStep = self.setExecutable('./dirac_prod3_evndisp_final',
                                arguments="--prefix %s --layout_list '%s' \
                                    --focal_file %s --calibration_file %s \
                                    --reconstructionparameter %s  --taskid %s" %
                                (self.prefix, self.layout_list,
                                 self.focal_file, self.calibration_file,
                                 self.reconstructionparameter, self.ts_task_id),
                                logFile='EvnDisp_Log.txt')
    evStep['Value']['name'] = 'Step%i_EvnDisplay' % iStep
    evStep['Value']['descr_short'] = 'Run EvnDisplay'
    iStep += 1

    # step 4
    # ## the order of the metadata dictionary is important, since it's used to build the directory structure
    mdjson = json.dumps(self.metadata)

    metadatafield = {'array_layout': 'VARCHAR(128)', 'site': 'VARCHAR(128)',
                     'particle': 'VARCHAR(128)',
                     'phiP': 'float', 'thetaP': 'float',
                     self.program_category + '_prog': 'VARCHAR(128)',
                             self.program_category + '_prog_version': 'VARCHAR(128)',
                     'data_level': 'int', 'configuration_id': 'int'}
    mdfieldjson = json.dumps(metadatafield)

    # register Data
    outputpattern = './*evndisp-DL%01d.tar.gz' % self.output_data_level
    file_md_json = json.dumps(self.filemetadata)
    scripts = '../CTADIRAC/Core/scripts'
    dmStep = self.setExecutable(scripts + '/cta-analysis-managedata.py',
                                arguments="'%s' '%s' '%s' %s '%s' %s %s '%s'" %
                                (mdjson, mdfieldjson, file_md_json, self.basepath,
                                 outputpattern, self.package,
                                 self.program_category, self.catalogs),
                                logFile='DataManagement_Log.txt')
    dmStep['Value']['name'] = 'Step%i_DataManagement' % iStep
    dmStep['Value']['descr_short'] = 'Save files to SE and register them in DFC'
    iStep += 1

    # register Log
    outputpattern = './*.logs.tgz'
    filemetadata = {}
    file_md_json = json.dumps(filemetadata)
    dmStep = self.setExecutable('../CTADIRAC/Core/scripts/cta-analysis-managedata.py',
                                arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log" %
                                (mdjson, mdfieldjson, file_md_json, self.basepath,
                                 outputpattern, self.package, self.program_category, self.catalogs),
                                logFile='Log_DataManagement_Log.txt')
    dmStep['Value']['name'] = 'Step%i_Log_DataManagement' % iStep
    dmStep['Value']['descr_short'] = 'Save log files to SE and register them in DFC'
    iStep += 1

    # step 6 -- to be removed -- debug only
    if debug:
      lsStep = self.setExecutable('/bin/ls',
                                  arguments=" -alhtr; /bin/ls -lahrt ./Data",
                                  logFile='LS_End_Log.txt')
      lsStep['Value']['name'] = 'Step%i_LS_End' % iStep
      lsStep['Value']['descr_short'] = 'list files in working directory and in Data directory'
      iStep += 1



""" Helper module that contains a set of useful simple and generic functions

                    JB, September 2018
"""

__RCSID__ = "$Id$"

import os
import re
import copy
import datetime

import DIRAC
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from DIRAC.Core.Utilities.ReturnValues import returnSingleResult


# Jobs status dictionnary
BASE_STATUS_DIR = {'Received': 0, 'Matched': 0, 'Waiting': 0, 'Running': 0,
                   'Failed': 0, 'Stalled': 0, 'Rescheduled': 0, 'Checking': 0,
                   'Done': 0, 'Completing': 0, 'Completed': 0, 'Killed': 0,
                   'Total': 0}

# Data level meta data id
DATA_LEVEL_METADATA_ID = {'MC0': -3, 'R1': -2, 'R0': -1,
                          'DL0': 0, 'DL1': 1, 'DL2': 2, 'DL3': 3, 'DL4': 4, 'DL5': 5}


def highlight(string):
  ''' highlight a string in a terminal display
  '''
  return '\x1b[31;1m%s\x1b[0m' % (string)


def read_inputs_from_file(file_path):
  """ Read inputs from an ASCII files
      Used for list of LFN or list of datasets
      Expects just one LFN/dataset per line
  """
  content = open(file_path, 'rt').readlines()
  input_file_list = []
  for line in content:
    infile = line.strip()
    if line != "\n":
      input_file_list.append(infile)
  return input_file_list


def run_number_from_filename(filename, package):
  """ try to get a run number from the file name

  return:
      run_number : int - the run number
  """
  run_number = -1
  if filename[-9:] == '.logs.tgz':
    run_number = int(filename.split('/')[-1].split('_')[-1].split('.')[0])
  elif package in ['chimp', 'mars']:
    run_number = int(filename.split('run')[1].split('___cta')[0])
  elif package in ['corsika_simhessarray', 'corsika_simtelarray']:
    if filename[-12:] in ['.corsika.zst']:
      # int(re.findall(r'run\d+_', os.path.basename(filename))[0].strip('run_'))
      run_number = int(os.path.basename(filename).split('_')[0].strip('run'))
    elif filename.find('tid') > 0:
      run_number = int(re.findall(r'tid\d+', os.path.basename(filename))[0].strip('tid'))
    elif os.path.splitext(filename)[1] in ['.log']:
      run_number = int(os.path.basename(filename).strip('run.log'))
    else:
      run_number = int(filename.split('run')[1].split('___cta')[0])
  elif package == 'evndisplay':
    if filename.find('tid') > 0:
      run_number = int(re.findall(r'tid\d+', os.path.basename(filename))[0].strip('tid'))
    elif filename[-8:] in ['DL1.root', 'DL2.root']:
      run_number = int(filename.split('run')[1].split('___cta')[0])
    elif filename[-10:] in ['DL1.tar.gz', 'DL2.tar.gz']:
      run_number = int(filename.split('run')[1].split('___cta')[0])
    else:
      run_number = int(filename.split('-')[0])  # old default
  elif package == 'image_extractor':
    run_number = int(filename.split('srun')[1].split('-')[0])
  elif package == 'dl1_data_handler':
    run_number = int(filename.split('runs')[1].split('-')[0])
  elif package == 'ctapipe':
    run_number = int(filename.split('run')[1].split('___cta')[0])
  return run_number


def check_dataset_query(dataset_name):
  """ print dfind command for a given dataset
  """
  md_dict = get_dataset_MQ(dataset_name)
  return debug_query(md_dict)


def debug_query(MDdict):
  """ just unwrap a meta data dictionnary into a dfind command
  """
  msg = 'dfind /vo.cta.in2p3.fr/MC/'
  for key, val in MDdict.items():
    try:
      val = val.values()[0]
    except BaseException:
      pass
    msg += ' %s=%s' % (key, val)
  return msg


def get_dataset_MQ(dataset_name):
  """ Return the Meta Query associated with a given dataset
  """
  fc = FileCatalogClient()
  result = returnSingleResult(fc.getDatasetParameters(dataset_name))
  if not result['OK']:
    DIRAC.gLogger.error("Failed to retrieved dataset:",
                        result['Message'])
    DIRAC.exit(-1)
  else:
    DIRAC.gLogger.info("Successfully retrieved dataset: ", dataset_name)
  return result['Value']['MetaQuery']


def get_job_list(owner, job_group, n_hours):
  ''' get a list of jobs for a selection
  '''
  from DIRAC.Interfaces.API.Dirac import Dirac
  dirac = Dirac()

  now = datetime.datetime.now()
  onehour = datetime.timedelta(hours=1)
  results = dirac.selectJobs(
      jobGroup=job_group,
      owner=owner,
      date=now - n_hours * onehour)
  if 'Value' not in results:
    DIRAC.gLogger.error(
        "No job found for group \"%s\" and owner \"%s\" in the past %s hours" %
        (job_group, owner, n_hours))
    DIRAC.exit(-1)

  # Found some jobs, print information)
  jobs_list = results['Value']
  return jobs_list


def parse_jobs_list(jobs_list):
  ''' parse a jobs list by first getting the status of all jobs
  '''
  from DIRAC.Interfaces.API.Dirac import Dirac
  dirac = Dirac()
  # status of all jobs
  status = dirac.getJobStatus(jobs_list)
  # parse it
  sites_dict = {}
  status_dict = copy.copy(BASE_STATUS_DIR)
  for job in jobs_list:
    site = status['Value'][int(job)]['Site']
    minstatus = status['Value'][int(job)]['MinorStatus']
    majstatus = status['Value'][int(job)]['Status']
    if majstatus not in status_dict.keys():
      DIRAC.gLogger.notice('Add %s to BASE_STATUS_DIR' % majstatus)
      DIRAC.sys.exit(1)
    status_dict[majstatus] += 1
    status_dict['Total'] += 1
    if site not in sites_dict.keys():
      if site.find('.') == -1:
        site = '    None'  # note that blank spaces are needed
      sites_dict[site] = copy.copy(BASE_STATUS_DIR)
      sites_dict[site][majstatus] = 1
      sites_dict[site]["Total"] = 1
    else:
      sites_dict[site]["Total"] += 1
      if majstatus not in sites_dict[site].keys():
        sites_dict[site][majstatus] = 1
      else:
        sites_dict[site][majstatus] += 1
  return status_dict, sites_dict

def get_os_and_cpu_info():
  ''' get OS and instructions supported by current cpu
  '''
  import platform
  import subprocess
  import re
  platform = platform.platform()
  cpuinfo = subprocess.check_output('cat /proc/cpuinfo', shell=True, universal_newlines=True)
  model_name = subprocess.check_output('grep -m 1 "model name" /proc/cpuinfo', shell=True, universal_newlines=True).strip()
  for inst in ['avx512', 'avx2', 'avx', 'sse4']:
    if re.search(inst, cpuinfo) is not None:
      break
  return (platform, model_name, inst)



#!/usr/bin/env python
""" General script to setup software through the SoftwareManager module
        J. Bregeon, L. Arrabito 15/09/2019
"""

__RCSID__ = "$Id$"

# generic imports
import os

# DIRAC imports
import DIRAC
# To do: change import
from DIRAC.Core.Base import Script

Script.registerSwitch("p:", "Package=", "Software package name")
Script.registerSwitch("v:", "Version=", "Base version to look for")
Script.registerSwitch("a:", "Category=", "Program category (simulations, analysis...)")
Script.registerSwitch("g:", "Compiler=", "Target a compiler_optimization configuration")
Script.registerSwitch("r:", "Repository=", "Target CVMFS repository")

Script.setUsageMessage('\n'.join([__doc__.split('\n')[1],
                                  'Usage:',
                                  '  %s -p package -v version -a [program_category] -g [compiler]' % Script.scriptName,
                                  'Arguments:',
                                  '  package: corsika_simtelarray',
                                  '  version: 2019-09-03',
                                  '  program_category: simulations',
                                  '  compiler: gcc48_avx2',
                                  '\ne.g: %s -p corsika_simtelarray -v 2019-09-03' % Script.scriptName,
                                  ]))

Script.parseCommandLine(ignoreErrors=False)


# Specific DIRAC imports
from CTADIRAC.Core.Utilities.SoftwareManager import SoftwareManager


def main():
  """ setup a given software package to be used in the main workflow

  Keyword arguments:
      package: software name
      version: software version
      category: simulations, analysis...
      compiler: compiler and configuration
  """
  package = None
  version = None
  category = 'simulations'
  compiler = 'gcc48_default'
  repository = None
  for switch in Script.getUnprocessedSwitches():
    if switch[0] == "p" or switch[0].lower() == "package":
      package = switch[1]
    if switch[0] == "v" or switch[0].lower() == "version":
      version = switch[1]
    if switch[0] == "a" or switch[0].lower() == "category":
      category = switch[1]
    if switch[0] == "g" or switch[0].lower() == "compiler":
      compiler = switch[1]
    if switch[0] == "r" or switch[0].lower() == "repository":
      repository = switch[1]
  if package is None or version is None:
    DIRAC.gLogger.error('Please give a package name and a version')
    DIRAC.exit(-1)
  DIRAC.gLogger.notice('Trying to setup: %s %s %s %s'
                       % (package, version, category, compiler))
  # get arguments
  soft_category = {package: category}
  manager = SoftwareManager(soft_category)
  # for testing only
  if repository is not None:
    manager.CVMFS_DIR = repository
  # check if and where Package is available
  # return cvmfs/tarball and full path
  res = manager.find_software(package, version, compiler)
  if not res['OK']:  # could not find package
    return res
  source = res['Value']['Source']
  package_dir = res['Value']['Path']
  if source is 'cvmfs':
    res = manager.install_dirac_scripts(package_dir)
    if not res['OK']:
      return res
    res = manager.dump_setup_script_path(package_dir)
    if not res['OK']:
      return res
  elif source is 'tarball':
    res = manager.install_software(os.path.join(package_dir, package + '.tar.gz'))
    if not res['OK']:
      return res
    package_local_path = res['Value']
    res = manager.dump_setup_script_path(package_local_path)
    if not res['OK']:
      return res
  DIRAC.gLogger.notice('Setup software completed successfully')
  DIRAC.exit()


####################################################
if __name__ == '__main__':
  main()



""" EvnDisp Script to create a Transformation with Input Data
    version created to run the Final version of EventDisplay
    https://forge.in2p3.fr/issues/27528
                        February 16th 2018 - J. Bregeon
"""

__RCSID__ = "$Id$"

import json
from copy import copy

from DIRAC.Core.Base import Script
Script.setUsageMessage('\n'.join([__doc__.split('\n')[1],
                                  'Usage:',
                                  '  %s trans_name input_dataset_name group_size' % Script.scriptName,
                                  'Arguments:',
                                  '  trans_name: name of the transformation',
                                  '  input_dataset_name: name of the input dataset',
                                  '  group_size: n files to process',
                                  '\ne.g: %s evndisp-gamma-N Paranal_gamma_North_20deg_HB9' % Script.scriptName,
                                  ]))

Script.parseCommandLine()

import DIRAC
from DIRAC.TransformationSystem.Client.Transformation import Transformation
from CTADIRAC.Interfaces.API.EvnDisp3FinalJob import EvnDisp3FinalJob
from DIRAC.Core.Workflow.Parameter import Parameter
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from DIRAC.Core.Utilities.ReturnValues import returnSingleResult


def check_dataset_query(dataset_name):
  """ print dfind command for a given dataset
  """
  md_dict = get_dataset_MQ(dataset_name)
  return debug_query(md_dict)


def debug_query(MDdict):
  """ just unwrap a meta data dictionnary into a dfind command
  """
  msg = 'dfind /vo.cta.in2p3.fr/MC/'
  for key, val in MDdict.items():
    try:
      val = val.values()[0]
    except BaseException:
      pass
    msg += ' %s=%s' % (key, val)
  return msg


def get_dataset_MQ(dataset_name):
  """ Return the Meta Query associated with a given dataset
  """
  fc = FileCatalogClient()
  result = returnSingleResult(fc.getDatasetParameters(dataset_name))
  if not result['OK']:
    DIRAC.gLogger.error("Failed to retrieved dataset:",
                        result['Message'])
    DIRAC.exit(-1)
  else:
    DIRAC.gLogger.info("Successfully retrieved dataset: ", dataset_name)
  return result['Value']['MetaQuery']


def submit_trans(job, transName, mqJson, group_size):
  """ Create a transformation executing the job workflow
  """
  DIRAC.gLogger.notice('submit_trans : %s' % transName)

  # Initialize JOB_ID
  job.workflow.addParameter(Parameter("JOB_ID", "000000", "string", "", "",
                                      True, False, "Temporary fix"))

  trans = Transformation()
  trans.setTransformationName(transName)  # this must be unique
  trans.setType("DataReprocessing")
  trans.setDescription("EvnDisplay MQ example")
  trans.setLongDescription("EvnDisplay calib_imgreco")  # mandatory
  trans.setBody(job.workflow.toXML())
  trans.setGroupSize(group_size)
  trans.setFileMask(mqJson)  # catalog query is defined here
  trans.addTransformation()  # transformation is created here
  trans.setStatus("Active")
  trans.setAgentType("Automatic")
  return trans


def runEvnDisp3MQ(args=None):
  """ Simple wrapper to create a EvnDisp3RefJob and setup parameters
      from positional arguments given on the command line.

      Parameters:
      args -- infile mode
  """
  DIRAC.gLogger.notice('runEvnDisp3MQ')
  # get arguments
  transName = args[0]
  dataset_name = args[1]
  group_size = int(args[2])

  ################################
  job = EvnDisp3FinalJob(cpuTime=432000)  # to be adjusted!!

  ### Main Script ###
  # override for testing
  job.setName('EvnDisp3')
  # add for testing
  job.setType('EvnDisp3')

  # change here for Paranal or La Palma
  job.version = 'prod3b_d20180201-lp'
  job.prefix = "CTA.prod3Nb"
  job.calibration_file = 'prod3b.Paranal-20171214.ped.root'

  # get input data set meta query
  # MDdict = {'MCCampaign':'PROD3', 'particle':particle,
  #           'array_layout':'full', 'site':'Paranal',
  #           'outputType':'Data', 'thetaP':{"=": 20}, 'phiP':{"=": 180.0},
  #           'tel_sim_prog':'simtel', 'tel_sim_prog_version':'2016-06-28',
  #           'sct'=False}
  input_meta_query = get_dataset_MQ(dataset_name)
  # input_meta_query['sct']='False'
  # process SCT arrays for 20 deg
  if input_meta_query['thetaP'] == 20:
    job.layout_list = '3AL4-BF15 3AL4-BN15 3AL4-BS15 3AL4-BN15-TI 3AL4-BF15-TI 3AL4-BS15-TI'
  else:
    job.layout_list = '3AL4-BF15 3AL4-BN15 3AL4-BN15-TI 3AL4-BF15-TI'

  # refine output meta data if needed
  output_meta_data = copy(input_meta_query)
  job.setEvnDispMD(output_meta_data)

  # add the sequence of executables
  job.ts_task_id = '@{JOB_ID}'  # dynamic
  job.setupWorkflow(debug=False)

  # output
  job.setOutputSandbox(['*Log.txt'])

  # submit the workflow to the TS
  res = submit_trans(job, transName, json.dumps(input_meta_query), group_size)

  return res


#########################################################
if __name__ == '__main__':

  args = Script.getPositionalArgs()
  if (len(args) != 3):
    Script.showHelp()

  runEvnDisp3MQ(args)



"""
  Simple Wrapper on the Job class to handle EvnDisp Analysis
  for the Prod4 SST run
  https://forge.in2p3.fr/issues/35197
                        October 31st 2018 - J. Bregeon
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections
# DIRAC imports
from DIRAC.Interfaces.API.Job import Job
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from CTADIRAC.Core.Utilities.tool_box import DATA_LEVEL_METADATA_ID


class EvnDisp4SSTJob(Job):
  """ Job extension class for EvnDisp Analysis,
    takes care of running converter and evndisp.
  """

  def __init__(self, cpuTime=432000):
    """ Constructor

    Keyword arguments:
    cpuTime -- max cpu time allowed for the job
    """
    Job.__init__(self)
    self.setCPUTime(cpuTime)
    # defaults
    self.setName('Evndisplay_CalibReco')
    self.package = 'evndisplay'
    self.program_category = 'calibimgreco'
    self.version = 'prod4_d20181110'
    self.configuration_id = 4
    self.output_data_level = DATA_LEVEL_METADATA_ID['DL1']
    self.N_output_files = 1
    self.prefix = 'CTA.prod4S'
    self.layout = '3HB9-SST'
    self.calibration_file = 'prod4b-SST-IPR.root'
    self.reconstructionparameter = 'EVNDISP.prod4.reconstruction.runparameter.NN.noLL'
    self.base_path = '/vo.cta.in2p3.fr/MC/PROD4/'
    self.fcc = FileCatalogClient()
    self.metadata = collections.OrderedDict()
    self.filemetadata = {}
    self.catalogs = json.dumps(['DIRACFileCatalog', 'TSCatalog'])
    self.ts_task_id = 0

  def set_meta_data(self, simtel_md):
    """ Set evndisplay meta data

    Parameters:
    simtel_md -- metadata dictionary as for simtel
    """
    # # Set evndisp directory meta data
    self.metadata['array_layout'] = simtel_md['array_layout']
    self.metadata['site'] = simtel_md['site']
    self.metadata['particle'] = simtel_md['particle']
    try:
      phiP = simtel_md['phiP']['=']
    except BaseException:
      phiP = simtel_md['phiP']
    self.metadata['phiP'] = phiP
    try:
      thetaP = simtel_md['thetaP']['=']
    except BaseException:
      thetaP = simtel_md['thetaP']
    self.metadata['thetaP'] = thetaP
    self.metadata[self.program_category + '_prog'] = 'evndisp'
    self.metadata[self.program_category + '_prog_version'] = self.version
    self.metadata['data_level'] = self.output_data_level
    self.metadata['configuration_id'] = self.configuration_id

  def set_file_meta_data(self, simtel_md):
    """ Set evndisplay file meta data

    Parameters:
    simtel_md -- metadata dictionary as for simtel
    """
    # # Set evndisp directory meta data
    self.filemetadata['tel_config'] = simtel_md['tel_config']

  def setupWorkflow(self, debug=False):
    """ Setup job workflow by defining the sequence of all executables
        All parameters shall have been defined before that method is called.
    """
    # step 1 -- debug
    iStep = 1
    if debug:
      lsStep = self.setExecutable('/bin/ls -alhtr',
                                  logFile='LS_Init_Log.txt')
      lsStep['Value']['name'] = 'Step%i_LS_Init' % iStep
      lsStep['Value']['descr_short'] = 'list files in working directory'
      iStep += 1

    # step 2
    swStep = self.setExecutable('cta-prod3-setupsw',
                                arguments='%s %s' % (self.package,
                                                     self.version),
                                logFile='SetupSoftware_Log.txt')
    swStep['Value']['name'] = 'Step%i_SetupSoftware' % iStep
    swStep['Value']['descr_short'] = 'Setup software'
    iStep += 1

    # step 3
    evStep = self.setExecutable('./dirac_prod4_sst_evndisp',
                                arguments="--prefix %s --layout '%s' \
                                    --calibration_file %s \
                                    --reconstructionparameter %s  --taskid %s" %
                                (self.prefix, self.layout,
                                 self.calibration_file,
                                 self.reconstructionparameter, self.ts_task_id),
                                logFile='EvnDisp_Log.txt')
    evStep['Value']['name'] = 'Step%i_EvnDisplay' % iStep
    evStep['Value']['descr_short'] = 'Run EvnDisplay'
    iStep += 1

    # step 4
    # ## the order of the metadata dictionary is important, since it's used to build the directory structure
    md_json = json.dumps(self.metadata)
    meta_data_field = {'array_layout': 'VARCHAR(128)', 'site': 'VARCHAR(128)',
                       'particle': 'VARCHAR(128)',
                       'phiP': 'float', 'thetaP': 'float',
                       self.program_category + '_prog': 'VARCHAR(128)',
                               self.program_category + '_prog_version': 'VARCHAR(128)',
                       'data_level': 'int', 'configuration_id': 'int'}
    md_field_json = json.dumps(meta_data_field)

    # register Data
    output_pattern = './*evndisp-*-DL%01d.tar.gz' % self.output_data_level
    file_md_json = json.dumps(self.filemetadata)
    scripts = '../CTADIRAC/Core/scripts/'
    dm_step = self.setExecutable(scripts + 'cta-prod-managedata.py',
                                 arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data" %
                                 (md_json, md_field_json, file_md_json,
                                  self.base_path, output_pattern, self.package,
                                  self.program_category, self.catalogs),
                                 logFile='DataManagement_Log.txt')
    dm_step['Value']['name'] = 'Step%i_DataManagement' % iStep
    dm_step['Value']['descr_short'] = 'Save files to SE and register them in DFC'
    iStep += 1

    # register Log
    log_file_pattern = './*.logs.tgz'
    filemetadata = {}
    file_md_json = json.dumps(filemetadata)
    scripts = '../CTADIRAC/Core/scripts/'
    log_step = self.setExecutable(scripts + 'cta-prod-managedata.py',
                                  arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log" %
                                  (md_json, md_field_json, file_md_json,
                                   self.base_path, log_file_pattern, self.package,
                                   self.program_category, self.catalogs),
                                  logFile='LogManagement_Log.txt')
    log_step['Value']['name'] = 'Step%i_Log_DataManagement' % iStep
    log_step['Value']['descr_short'] = 'Save log files to SE and register them in DFC'
    iStep += 1

    # step 6 -- to be removed -- debug only
    if debug:
      lsStep = self.setExecutable('/bin/ls -alhtr',
                                  logFile='LS_End_Log.txt')
      lsStep['Value']['name'] = 'Step%i_LS_End' % iStep
      lsStep['Value']['descr_short'] = 'list files in working directory and in Data directory'
      iStep += 1



""" EvnDisp Script to create a Transformation with Input Data
"""

import json
from DIRAC.Core.Base import Script
Script.setUsageMessage('\n'.join([__doc__.split('\n')[1],
                                  'Usage:',
                                  '  %s transName' % Script.scriptName,
                                  'Arguments:',
                                  '  transName: name of the transformation',
                                  '\ne.g: %s evndisp-gamma-N' % Script.scriptName,
                                  ]))

Script.parseCommandLine()

import DIRAC
from DIRAC.TransformationSystem.Client.Transformation import Transformation
from CTADIRAC.Interfaces.API.EvnDisp3MSCWRefJob import EvnDisp3MSCWRefJob
from DIRAC.Core.Workflow.Parameter import Parameter


def submitTS(job, transName, mqJson):
  """ Create a transformation executing the job workflow
  This is using a file mask so that files be added on the fly.
  """
  DIRAC.gLogger.notice('submitTS')

  # Initialize JOB_ID
  job.workflow.addParameter(Parameter("JOB_ID", "000000", "string", "", "",
                                      True, False, "Temporary fix"))

  t = Transformation()
  t.setTransformationName(transName)  # this must be unique
  t.setType("DataReprocessing")
  t.setDescription("EvnDisp3MSCW example")
  t.setLongDescription("EvnDisplay stereo reconstruction")  # mandatory
  t.setBody(job.workflow.toXML())
  t.setGroupSize(100)
  t.setFileMask(mqJson)  # catalog query is defined here
  t.addTransformation()  # transformation is created here
  t.setStatus("Active")
  t.setAgentType("Automatic")
  trans_id = t.getTransformationID()
  return trans_id


def runEvnDisp3MSCW(args=None):
  """ Simple wrapper to create a EvnDisp3RefJob and setup parameters
      from positional arguments given on the command line.

      Parameters:
      args -- infile mode
  """
  DIRAC.gLogger.notice('runEvnDisp3MSCW')
  # get arguments
  transName = args[0]

  ################################
  job = EvnDisp3MSCWRefJob(cpuTime=432000)  # to be adjusted!!

  ### Main Script ###
  # override for testing
  job.setName('EvnDisp3MSCW')
  # add for testing
  job.setType('EvnDisp3')

  # defaults
  # job.setLayout( "Baseline")
  ## package and version
  # job.setPackage( 'evndisplay' )
  # job.setVersion( 'prod3b_d20170602' )

  # change here for Paranal or La Palma
  job.setPrefix("CTA.prod3Nb")
  job.setPointing(180)  # must match table
  job.setDispSubDir('BDTdisp.Nb.3AL4-BN15.T1')
  job.setRecId('0,1,2')  # 0 = all teltescopes, 1 = LST only, 2 = MST only
  #  set calibration file and parameters file, must match pointing
  job.setTableFile('tables_CTA-prod3b-LaPalma-NNq05-NN-ID0_180deg-d20160925m4-Nb.3AL4-BN15.root')

  # set meta-data to the product of the transformation
  # set query to add files to the transformation
  MDdict = {'MCCampaign': 'PROD3', 'particle': 'gamma',
            'array_layout': 'Baseline', 'site': 'LaPalma',
            'outputType': 'Data', 'data_level': {"=": 1},
            'configuration_id': {"=": 0},
            'calibimgreco_prog': 'evndisp',
            'calibimgreco_prog_version': 'prod3b_d20170602',
            'thetaP': {"=": 20}, 'phiP': {"=": 0.0}}
  job.setEvnDispMD(MDdict)

  # add the sequence of executables
  job.setTSTaskId('@{JOB_ID}')  # dynamic
  job.setupWorkflow(debug=True)

  # output
  job.setOutputSandbox(['*Log.txt'])

  # submit the workflow to the TS
  res = submitTS(job, transName, json.dumps(MDdict))

  return res


#########################################################
if __name__ == '__main__':

  args = Script.getPositionalArgs()
  if (len(args) != 1):
    Script.showHelp()

  res = runEvnDisp3MSCW(args)
  if not res['OK']:
    DIRAC.gLogger.error(res['Message'])
    DIRAC.exit(-1)
  else:
    DIRAC.gLogger.notice('Done')



#!/usr/bin/env python
""" Simple data management script for PROD3 MC
    create DFC MetaData structure
    put and register files in DFC
"""

__RCSID__ = "$Id$"

# generic imports
import os
import glob
import json

# DIRAC imports
import DIRAC
from DIRAC.Core.Base import Script
Script.parseCommandLine()

# Specific DIRAC imports
from CTADIRAC.Core.Workflow.Modules.ProdDataManager import ProdDataManager

####################################################
def main():
    """ simple wrapper to put and register all PROD3 files
    
    Keyword arguments:
    args -- a list of arguments in order []
    """
    args = Script.getPositionalArgs()
    metadata = args[0] 
    metadatafield = args[1]
    filemetadata = args[2]
    inputpath = args[3]
    basepath = args[4]
    start_run_nb = args[5]
    catalogs = args[6]
    catalogsjson = json.loads( catalogs )
    
    # # Create MD structure
    prod3dm=ProdDataManager( catalogsjson )
    res = prod3dm.createMDStructure( metadata, metadatafield, basepath, 'tel_sim' )
    if res['OK']:
      path = res['Value']
    else:
      return res

    # # Upload data files
    datadir = os.path.join( inputpath, 'Data/*' )
    res = prod3dm._checkemptydir( datadir )
    if not res['OK']:
      return res

    # ## Add start_run_number to run_number and update filemetadata
    fmd = json.loads( filemetadata )
    run_number = int( fmd['runNumber'] ) + int( start_run_nb )
    fmd['runNumber'] = '%08d' % run_number
    fmdjson = json.dumps( fmd )

    # ## Get the run directory
    runpath = prod3dm._getRunPath( fmd )

    for localfile in glob.glob( datadir ):
      filename = os.path.basename( localfile )
      lfn = os.path.join( path, 'Data', runpath, filename )
      res = prod3dm.putAndRegister( lfn, localfile, fmdjson )
      if not res['OK']:
        return res

    # ## Upload log files
    tarname = filename.split( '___cta-prod3' )[0] + '.tar.gz'
    res = prod3dm.createTarLogFiles( inputpath, tarname )
    if not res['OK']:
      return DIRAC.S_ERROR( 'prod3dm.createTarLogFiles failed' )
    lfn = os.path.join( path, 'Log', runpath, tarname )
    res = prod3dm.putAndRegister( lfn, tarname, fmdjson )
    if not res['OK']:
      return res

    DIRAC.exit()

####################################################
if __name__ == '__main__':
  main()



"""
    Launcher script to launch a production with 3 steps
    simulation Step: Prod5bMCPipeNSBJob
    processing Step: Prod5CtaPipeModelingJob
    merging step : Prod5CtaPipeMergeJob
"""

__RCSID__ = "$Id$"

import json
from copy import deepcopy

from DIRAC.Core.Base import Script

Script.setUsageMessage(
    "\n".join(
        [
            __doc__.split("\n")[1],
            "Usage:",
            "python %s.py <mode> <site> <particle> <pointing> <zenith> <n showers>"
            % Script.scriptName,
            "Arguments:",
            "  name: name of the production",
            "  site: either Paranal or LaPalma",
            "  particle: in gamma, gamma-diffuse, electron, proton",
            "  pointing: North or South",
            "  zenith: 20, 40 or 60",
            "  n shower: 100 for testing",
            "\ne.g: python %s.py Merge1 Paranal gamma North 20 100" % Script.scriptName,
        ]
    )
)
Script.parseCommandLine()
import DIRAC

from DIRAC.ProductionSystem.Client.ProductionClient import ProductionClient
from DIRAC.ProductionSystem.Client.ProductionStep import ProductionStep
from CTADIRAC.Interfaces.API.Prod5bMCPipeNSBJob import Prod5bMCPipeNSBJob
from CTADIRAC.Interfaces.API.Prod5CtaPipeModelingJob import Prod5CtaPipeModelingJob
from CTADIRAC.Interfaces.API.Prod5CtaPipeMergeJob import Prod5CtaPipeMergeJob


def define_meta_data_DL0(args):
    """Return meta query"""
    if args[3] == "North":
        phiP = 180.0
    elif args[3] == "South":
        phiP = 0

    dl0_meta_data = {
        "array_layout": "Advanced-Baseline",
        "site": args[1],
        "particle": args[2],
        "phiP": phiP,
        "thetaP": args[4],
        "data_level": 0,
        "nsb": 1,
        "outputType": "Data",
        "MCCampaign": "ProdTestAF"
    }
    return dl0_meta_data


def build_simulation_step(dl0_meta_data, args):
    """Setup MC simulation step

    @return ProductionStep object
    """
    prod_step_1 = ProductionStep()
    prod_step_1.Name = "MC_Simulation"
    DIRAC.gLogger.notice("\tBuilding Processing Production step: %s" % prod_step_1.Name)
    prod_step_1.Type = "MCSimulation"
    prod_step_1.Outputquery = dl0_meta_data

    # job setup
    job1 = Prod5bMCPipeNSBJob()  # to be adjusted!!
    job1.version = "2020-06-29b"
    job1.compiler = "gcc83_matchcpu"
    # override for testing
    job1.setName("Prod5b_MC_Pipeline_NSB")
    # parameters from command line
    job1.set_site(args[1])
    job1.set_particle(args[2])
    job1.set_pointing_dir(args[3])
    job1.zenith_angle = args[4]
    job1.n_shower = args[5]

    # output
    job1.base_path = "/vo.cta.in2p3.fr/user/a/afaure/prod5b/"
    job1.setOutputSandbox(["*Log.txt"])
    job1.start_run_number = "0"
    job1.run_number = "@{JOB_ID}"  # dynamic
    job1.output_file_size = 10000  # 200 MB
    job1.setupWorkflow(debug=False)
    # Add the job description to the first ProductionStep
    prod_step_1.Body = job1.workflow.toXML()
    # return ProductionStep object
    return prod_step_1


def build_processing_step(dl0_meta_data):
    """Setup ctapipe step

    @return ProductionStep object
    """
    prod_step_2 = ProductionStep()
    prod_step_2.Name = "Processing_ctapipe"
    DIRAC.gLogger.notice("\tBuilding Processing Production step: %s" % prod_step_2.Name)
    prod_step_2.Type = "DataReprocessing"
    prod_step_2.Inputquery = dl0_meta_data

    # Here define the job description (i.e. Name, Executable, etc.)
    # to be associated to the first ProductionStep, as done when using the TS
    job2 = Prod5CtaPipeModelingJob(cpuTime=259200.0)
    job2.setName("Prod5_ctapipe_modeling")
    job2.setOutputSandbox(["*Log.txt"])
    job2.base_path = "/vo.cta.in2p3.fr/user/a/afaure/prod5b"

    output_meta_data = deepcopy(prod_step_2.Inputquery)
    job2.set_meta_data(output_meta_data)
    job2.set_file_meta_data(nsb=output_meta_data["nsb"])
    prod_step_2.Outputquery = job2.metadata

    # configuration
    # set site dependent config
    cta_site = prod_step_2.Inputquery["site"].lower()
    if cta_site == "paranal":
        job2.ctapipe_site_config = "prod5b_paranal_alpha_nectarcam.yml"
    elif cta_site == "lapalma":
        job2.ctapipe_site_config = "prod5b_lapalma_alpha.yml"

    job2.setupWorkflow(debug=False)
    job2.setType("Stage1Processing")  # mandatory *here*

    # Add the job description to the first ProductionStep
    prod_step_2.Body = job2.workflow.toXML()
    prod_step_2.GroupSize = 5  # have to match the above group size?

    # return ProductionStep object
    return prod_step_2


def build_merging_step(prod_step_2):
    """Merge files from different runs

    @return ProductionStep object
    """

    prod_step_3 = ProductionStep()
    prod_step_3.Name = "Merge"
    DIRAC.gLogger.notice("\tBuilding Merging Production step: %s" % prod_step_3.Name)
    prod_step_3.Type = "Merging"  # This corresponds to the Transformation Type
    prod_step_3.Inputquery = prod_step_2.Outputquery
    prod_step_3.Outputquery = deepcopy(prod_step_3.Inputquery)

    # Here define the job description to be associated to the second ProductionStep
    job3 = Prod5CtaPipeMergeJob(cpuTime=259200.0)
    job3.setName("Prod5_ctapipe_merging")
    job3.base_path = "/vo.cta.in2p3.fr/user/a/afaure/prod5b"

    # output
    job3.setOutputSandbox(["*Log.txt"])
    # refine output meta data if needed
    output_meta_data = deepcopy(prod_step_3.Inputquery)
    job3.set_meta_data(output_meta_data)
    job3.set_file_meta_data(nsb=1)

    job3.ts_task_id = "@{JOB_ID}"  # dynamic
    job3.setupWorkflow(debug=False)
    job3.setType("Merging")  # mandatory *here*
    prod_step_3.Body = job3.workflow.toXML()
    prod_step_3.GroupSize = 5  # number of files to merge
    # return ProductionStep object
    return prod_step_3


########################################################
if __name__ == "__main__":

    arguments = Script.getPositionalArgs()
    if len(arguments) != 6:
        Script.showHelp()
    ##################################
    # Create the production
    prod_name = "ProdTestAF%s" % arguments[0]
    DIRAC.gLogger.notice("Building new production: %s" % prod_name)
    prod_sys_client = ProductionClient()

    dl0_meta_data = define_meta_data_DL0(arguments)
    dl1_meta_data = deepcopy(dl0_meta_data)
    dl1_meta_data["data_level"] = 1
    # Define the first ProductionStep (MC simulation)
    prod_step_1 = build_simulation_step(dl0_meta_data, arguments)
    # Add the step to the production
    prod_sys_client.addProductionStep(prod_step_1)
    ##################################
    # Define the second ProductionStep (ctapipe)
    prod_step_2 = build_processing_step(dl0_meta_data)
    prod_step_2.ParentStep = prod_step_1
    # Add the step to the production
    prod_sys_client.addProductionStep(prod_step_2)

    # ##################################
    # # Define merging step and add them to the production
    prod_step_3 = build_merging_step(prod_step_2)
    prod_step_3.ParentStep = prod_step_2
    prod_sys_client.addProductionStep(prod_step_3)

    ##################################
    # Get the production description
    prod_description = prod_sys_client.prodDescription
    # Create the production
    DIRAC.gLogger.notice("Creating production.")
    res = prod_sys_client.addProduction(prod_name, json.dumps(prod_description))
    if not res["OK"]:
        DIRAC.gLogger.error(res["Message"])
        DIRAC.exit(-1)

    # Start the production, i.e. instantiate the transformation steps
    res = prod_sys_client.startProduction(prod_name)

    if not res["OK"]:
        DIRAC.gLogger.error(res["Message"])
        DIRAC.exit(-1)

    DIRAC.gLogger.notice("Production %s successfully created" % prod_name)



"""
    Launcher script to launch a production with 2 steps on a split dataset test/train
    processing Step: Prod5CtaPipeModelingJob
    merging step : Prod5CtaPipeMergeJob
"""

__RCSID__ = "$Id$"

import json
from copy import deepcopy

from DIRAC.Core.Base import Script

Script.setUsageMessage(
    "\n".join(
        [
            __doc__.split("\n")[1],
            "Usage:",
            "python %s.py <name> <dataset>" % Script.scriptName,
            "Arguments:",
            "  name: name of the production",
            "  input_dataset: name of the input dataset",
            "\ne.g: python %s.py Split_Merge ProdTestAF_LaPalma_AdvancedBaseline_gamma"
            % Script.scriptName,
        ]
    )
)
Script.parseCommandLine()
import DIRAC

from DIRAC.ProductionSystem.Client.ProductionClient import ProductionClient
from DIRAC.ProductionSystem.Client.ProductionStep import ProductionStep
from CTADIRAC.Interfaces.API.Prod5CtaPipeModelingJob import Prod5CtaPipeModelingJob
from CTADIRAC.Core.Utilities.tool_box import get_dataset_MQ
from CTADIRAC.Interfaces.API.Prod5CtaPipeMergeJob import Prod5CtaPipeMergeJob


def define_meta_query(dl0_data_set, split="train", merged=0):
    """Return meta query"""

    meta_query = get_dataset_MQ(dl0_data_set)

    meta_data = {
        "array_layout": meta_query["array_layout"],
        "site": meta_query["site"],
        "particle": meta_query["particle"],
        "phiP": meta_query["phiP"],
        "thetaP": meta_query["thetaP"],
        "configuration_id": {"=": 7},
        "outputType": "Data",
        "MCCampaign": "ProdTestAF",
        "stage1_prog_version": "v0.15.0",
        "merged": {"=": merged},
        "split": split
    }
    return meta_data


def build_processing_step(dl0_data_set, split="train"):
    """Setup ctapipe step

    @return ProductionStep object
    """

    if split == "train":
        DIRAC.gLogger.notice("Train Analysis")
        dl0_data_set_split = dl0_data_set + "_train"
    elif split == "test":
        DIRAC.gLogger.notice("Test Analysis")
        dl0_data_set_split = dl0_data_set + "_test"

    prod_step_1 = ProductionStep()
    prod_step_1.Name = "Processing_ctapipe"
    DIRAC.gLogger.notice("\tBuilding Processing Production step: %s" % prod_step_1.Name)
    prod_step_1.Type = "DataReprocessing"
    prod_step_1.Inputquery = get_dataset_MQ(dl0_data_set_split)
    # Here define the job description (i.e. Name, Executable, etc.)
    # to be associated to the first ProductionStep, as done when using the TS
    job1 = Prod5CtaPipeModelingJob(cpuTime=259200.0)
    job1.setName("Prod5_ctapipe_modeling")
    job1.setOutputSandbox(["*Log.txt"])
    job1.base_path = "/vo.cta.in2p3.fr/user/a/afaure/prod5b"

    output_meta_data = deepcopy(prod_step_1.Inputquery)
    job1.set_meta_data(output_meta_data)
    job1.set_file_meta_data(split=output_meta_data["split"])
    output_query = define_meta_query(dl0_data_set_split, split, merged=0)
    prod_step_1.Outputquery = output_query

    # configuration
    # set site dependent config
    cta_site = prod_step_1.Inputquery["site"].lower()
    if cta_site == "paranal":
        job1.ctapipe_site_config = "prod5b_paranal_alpha_nectarcam.yml"
    elif cta_site == "lapalma":
        job1.ctapipe_site_config = "prod5b_lapalma_alpha.yml"

    job1.setupWorkflow(debug=False)
    job1.setType("Stage1Processing")  # mandatory *here*

    # Add the job description to the first ProductionStep
    prod_step_1.Body = job1.workflow.toXML()
    prod_step_1.GroupSize = 5  # have to match the above group size?

    # return ProductionStep object
    return prod_step_1


def build_merging_step(dl0_data_set, group_size, split="train", merged=0):
    """Merge files from different runs from a split dataset

    @return ProductionStep object
    """

    prod_step_2 = ProductionStep()
    prod_step_2.Name = "Merge"
    DIRAC.gLogger.notice("\tBuilding Merging Production step: %s" % prod_step_2.Name)
    prod_step_2.Type = "Merging"  # This corresponds to the Transformation Type
    prod_step_2.Inputquery = define_meta_query(dl0_data_set, split, merged)

    # Here define the job description to be associated to the second ProductionStep
    job2 = Prod5CtaPipeMergeJob(cpuTime=259200.0)
    job2.setName("Prod5_ctapipe_merging")
    job2.base_path = "/vo.cta.in2p3.fr/user/a/afaure/prod5b"

    # output
    job2.setOutputSandbox(["*Log.txt"])
    # refine output meta data if needed
    output_meta_data = deepcopy(prod_step_2.Inputquery)
    job2.set_meta_data(output_meta_data)
    job2.set_file_meta_data(split=output_meta_data["split"])

    prod_step_2.Outputquery = define_meta_query(dl0_data_set, split, merged + 1)

    job2.setupWorkflow(debug=False)
    job2.setType("Merging")  # mandatory *here*
    prod_step_2.Body = job2.workflow.toXML()
    prod_step_2.GroupSize = group_size  # number of files to merge
    # return ProductionStep object
    return prod_step_2


########################################################
if __name__ == "__main__":
    args = Script.getPositionalArgs()
    if len(args) != 2:
        Script.showHelp()
    dl0_data_set = args[1]

    ##################################
    # Create the production
    prod_name = "ProdTestAF_split_%s" % args[0]
    DIRAC.gLogger.notice("Building new production: %s" % prod_name)
    prod_sys_client = ProductionClient()

    ##################################
    # Define the first ProductionStep (ctapipe with train data)
    prod_step_1 = build_processing_step(dl0_data_set, split="train")
    # Add the step to the production
    prod_sys_client.addProductionStep(prod_step_1)

    ##################################
    # Define the second ProductionStep (ctapipe with test data)
    prod_step_2 = build_processing_step(dl0_data_set, split="test")
    # Add the step to the production
    prod_sys_client.addProductionStep(prod_step_2)

    ###################################
    # Merging steps : 5 train files (group_size = 5) / 10 test files (group_size = 5 then 2)
    # Define merging step for train data and add it to the production
    prod_step_3 = build_merging_step(dl0_data_set, 5, split="train", merged=0)
    prod_step_3.ParentStep = prod_step_1
    prod_sys_client.addProductionStep(prod_step_3)
    ###################################
    # Define first merging step for test data and add it to the production
    prod_step_4 = build_merging_step(dl0_data_set, 5, split="test", merged=0)
    prod_step_4.ParentStep = prod_step_2
    prod_sys_client.addProductionStep(prod_step_4)
    # # Define second merging step for merged test data and add it to the production
    prod_step_5 = build_merging_step(dl0_data_set, 2, split="test", merged=1)
    prod_step_5.ParentStep = prod_step_4
    prod_sys_client.addProductionStep(prod_step_5)

    ##################################
    # Get the production description
    prod_description = prod_sys_client.prodDescription
    # Create the production
    DIRAC.gLogger.notice("Creating production.")
    res = prod_sys_client.addProduction(prod_name, json.dumps(prod_description))
    if not res["OK"]:
        DIRAC.gLogger.error(res["Message"])
        DIRAC.exit(-1)

    # Start the production, i.e. instantiate the transformation steps
    res = prod_sys_client.startProduction(prod_name)

    if not res["OK"]:
        DIRAC.gLogger.error(res["Message"])
        DIRAC.exit(-1)

    DIRAC.gLogger.notice("Production %s successfully created" % prod_name)



""" Prod3 MC Pipe Script to create a Transformation
          JB, LA March 2018
"""

__RCSID__ = "$Id$"

from DIRAC.Core.Base import Script
Script.setUsageMessage(
    '\n'.join(
        [
            __doc__.split('\n')[1],
            'Usage:',
            '  %s array_layout site particle pointing_dir zenith_angle nShower' %
            Script.scriptName,
            'Arguments:',
            '  array_layout: demo',
            '  site: Paranal or LaPalma',
            '  particle: gamma, proton, electron',
            '  pointing_dir: North or South',
            '  zenith_agle: 20',
            '  nShower: from 5 to 25000',
            '\ne.g: %s Baseline Paranal gamma South 20 5' %
            Script.scriptName,
        ]))

Script.parseCommandLine()

import DIRAC
from DIRAC import gLogger
from DIRAC.TransformationSystem.Client.Transformation import Transformation
from DIRAC.Core.Workflow.Parameter import Parameter
from CTADIRAC.Interfaces.API.Prod3MCPipeDivergentJob import Prod3MCPipeDivergentJob
# from Prod3MCPipeDivergentJob import Prod3MCPipeDivergentJob
from DIRAC.Interfaces.API.Dirac import Dirac


def submitTS(job):
  """ Create a transformation executing the job workflow  """

  # ## Temporary fix to initialize JOB_ID #######
  job.workflow.addParameter(Parameter("JOB_ID", "000000", "string", "", "",
                                      True, False, "Temporary fix"))
  job.workflow.addParameter(Parameter("PRODUCTION_ID", "000000", "string",
                                      "", "", True, False, "Temporary fix"))
  job.setType('MCSimulation')  # Used for the JobType plugin

  t = Transformation()
  # This must be unique. If not set it's asked in the prompt
  # t.setTransformationName("Prod3Exemple")
  t.setType("MCSimulation")
  t.setDescription("MC Prod3 BaseLine test")
  t.setLongDescription("corsika-simtel production")  # mandatory
  t.setBody(job.workflow.toXML())

  res = t.addTransformation()  # Transformation is created here

  if not res['OK']:
    gLogger.error(res['Message'])
    DIRAC.exit(-1)

  t.setStatus("Active")
  t.setAgentType("Automatic")

  return res


def submitWMS(job):
  """ Submit the job to the WMS
  """
  dirac = Dirac()
  res = dirac.submitJob(job)
  Script.gLogger.notice('Submission Result: ', res)
  return res


def runProd3(args=None):
  """ Simple wrapper to create a Prod3MCPipeDivergentJob and setup parameters
  from positional arguments given on the command line.

  Parameters:
  args -- a list of 6 strings corresponding to job arguments
  array_layout site particle pointing_dir zenith_angle nShower
  demo LaPalma  gamma South 20 1000
  """
  # get arguments
  layout = args[0]
  site = args[1]
  particle = args[2]
  pointing = args[3]
  zenith = args[4]
  nShower = args[5]
  div_cfg_id = args[6]

  # Job Wrapper
  job = Prod3MCPipeDivergentJob()

  # override for testing
  job.setName('Div_LaPalma_20deg_%s_%s' % (particle, div_cfg_id))

  # package and version
  # job.setPackage('corsika_simhessarray')
  # job.setVersion('2017-04-19-div')  # divergent pointing

  # layout, site, particle, pointing direction, zenith angle
  # Baseline, LaPalma,  gamma, South,  20
  job.setArrayLayout(layout)
  job.setSite(site)
  job.setParticle(particle)
  job.setPointingDir(pointing)

  # more attributes
  job.zenith_angle = zenith
  job.nShower = nShower
  job.start_run_number = '0'
  job.run_number = '@{JOB_ID}'  # dynamicly resolved

  # Divergent configuration ID
  job.div_cfg_id = div_cfg_id

  # get dirac log files
  job.setOutputSandbox(['*Log.txt'])

  # submit to the wms
#    job.run_number = '123'
#    job.setOutputData(['%s/Data/*.log.gz' % job.inputpath,
#                       '%s/Data/*.simtel.gz' % job.inputpath])
#    job.setupWorkflow(debug=True, register=False)
#    res = submitWMS(job)

  # submit to the Transformation System
  job.setupWorkflow(debug=False, register=True)
  res = submitTS(job)

  # debug
  Script.gLogger.info(job.workflow)

  return res


if __name__ == '__main__':
  args = Script.getPositionalArgs()
  if (len(args) != 7):
    Script.showHelp()
  try:
    res = runProd3(args)
    if not res['OK']:
      DIRAC.gLogger.error(res['Message'])
      DIRAC.exit(-1)
    else:
      DIRAC.gLogger.notice('Done')
  except Exception:
    DIRAC.gLogger.exception()
    DIRAC.exit(-1)



""" Launcher script to launch a Prod4CorsikaLSTMagicJob
on the WMS or create a Transformation.

    https://forge.in2p3.fr/issues/3492
                    JB, February 2019
"""

from DIRAC.Core.Base import Script
Script.setUsageMessage('\n'.join([__doc__.split('\n')[1],
                                  'Usage:',
                                  '  %s mode file_path (trans_name) group_size' % Script.scriptName,
                                  'Arguments:',
                                  '  mode: WMS for testing, TS for production',
                                  '  site: either Paranal or LaPalma',
                                  '  particle: in gamma, gamma-diffuse, electron, proton',
                                  '  pointing: North or South',
                                  '  zenith: 20, 40 or 60',
                                  '  n shower: 100 for testing',
                                  '\ne.g: python %s.py WMS LaPalma gamma North 20 100' % Script.scriptName,
                                  ]))

Script.parseCommandLine()

import DIRAC
from DIRAC.TransformationSystem.Client.Transformation import Transformation
from CTADIRAC.Interfaces.API.Prod4CorsikaLSTMagicJob import Prod4CorsikaLSTMagicJob
from DIRAC.Core.Workflow.Parameter import Parameter
from DIRAC.Interfaces.API.Dirac import Dirac


def submit_trans(job, trans_name):
  """ Create a transformation executing the job workflow
  """
  DIRAC.gLogger.notice('submit_trans : %s' % trans_name)

  # Initialize JOB_ID
  job.workflow.addParameter(Parameter("JOB_ID", "000000", "string", "", "",
                                      True, False, "Temporary fix"))

  trans = Transformation()
  trans.setTransformationName(trans_name)  # this must be unique
  trans.setType("MCSimulation")
  trans.setDescription("Prod4 Corsika TS")
  trans.setLongDescription("Prod4 Corsika LST-MAGIC simulation")  # mandatory
  trans.setBody(job.workflow.toXML())
  result = trans.addTransformation()  # transformation is created here
  if not result['OK']:
    return result
  trans.setStatus("Active")
  trans.setAgentType("Automatic")
  trans_id = trans.getTransformationID()
  return trans_id


def submit_wms(job):
  """ Submit the job to the WMS
  @todo launch job locally
  """
  dirac = Dirac()
  job.setJobGroup('Prod4CorsikaSSTJob')
  result = dirac.submitJob(job)
  if result['OK']:
    Script.gLogger.notice('Submitted job: ', result['Value'])
  return result


def run_corsika_sst(args):
  """ Simple wrapper to create a Prod4CorsikaLSTMagicJob and setup parameters
      from positional arguments given on the command line.

      Parameters:
      args -- mode (trans_name)
  """
  DIRAC.gLogger.notice('run_corsika_sst')
  # get arguments
  mode = args[0]

  # job setup
  job = Prod4CorsikaLSTMagicJob()
  # override for testing
  job.setName('Prod4_Corsika_LSTMagic')
  # parameters from command line
  job.set_site(args[1])
  job.set_particle(args[2])
  job.set_pointing_dir(args[3])
  job.zenith_angle = args[4]
  job.n_shower = args[5]

  # output
  job.setOutputSandbox(['*Log.txt'])

  # specific configuration
  if mode == 'WMS':
    job.base_path = '/vo.cta.in2p3.fr/user/b/bregeon'
    job.start_run_number = '1000'
    job.run_number = '31'
    job.setupWorkflow(debug=True)
    # subtmit to the WMS for debug
    job.setDestination('LCG.IN2P3-CC.fr')
    # job.setDestination('LCG.CIEMAT.es')
    result = submit_wms(job)
  elif mode == 'TS':
    job.base_path = '/vo.cta.in2p3.fr/MC/PROD4/'
#        job.start_run_number = '100000'
    job.run_number = '@{JOB_ID}'  # dynamic
    job.setupWorkflow(debug=False)
    job.setType('MCSimulation')
    tag = 'v2'
    trans_name = 'MC_Prod4_CorsikaLSTMagic_%s_%s_%s_%s%s' %\
        (job.cta_site, job.particle, job.pointing_dir, job.zenith_angle, tag)
    result = submit_trans(job, trans_name)
  else:
    DIRAC.gLogger.error('1st argument should be the job mode: WMS or TS,\n\
                             not %s' % mode)
    return None

  return result


#########################################################
if __name__ == '__main__':

  arguments = Script.getPositionalArgs()
  if len(arguments) != 6:
    Script.showHelp()
  try:
    result = run_corsika_sst(arguments)
    if not result['OK']:
      DIRAC.gLogger.error(result['Message'])
      DIRAC.exit(-1)
    else:
      DIRAC.gLogger.notice('Done')
  except Exception:
    DIRAC.gLogger.exception()
    DIRAC.exit(-1)



#!/usr/bin/env python

__RCSID__ = "$Id$"

# generic imports
import os
import six

# DIRAC imports
from DIRAC.Core.Base import Script
import DIRAC

Script.parseCommandLine(ignoreErrors=True)

from DIRAC import gLogger
from DIRAC.Interfaces.API.Dirac import Dirac
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient

def main():

  fcc = FileCatalogClient()
  dirac = Dirac()
  resJDL = dirac.getJobJDL(os.environ['JOBID'])

  idata = resJDL['Value']['InputData']
  if isinstance(idata, six.string_types):
    idata = []
    if 'LFN' in resJDL['Value']['InputData']:
      idata.append(resJDL['Value']['InputData'].split('LFN:')[1])
    else:
      idata.append(resJDL['Value']['InputData'])
  else:
    idata = resJDL['Value']['InputData']

  for lfn in idata:
    gLogger.notice("Checking input Log file:\n %s " % lfn)
    file_name = os.path.basename(lfn)

    # List the content of the tar file
    res = list(os.popen("tar -tf " + file_name + " '*log.gz'"))
    tardir = res[0].split('/')[0]
    # Extract the tar file
    code = os.system('tar -xvf ' + file_name)
    if code!=0:
       gLogger.error("Unable to untar Log file:\n %s" % file_name)
       DIRAC.exit(-1)
    os.system("gunzip -r " + tardir )
    res = list(os.popen("grep camera_CTA-LST -r " + tardir + " | grep file | wc -l"))
    if res[0].strip()!='4':
      data_lfn = lfn.replace("Log","Data").replace("log_hist.tar","simtel.zst")
      gLogger.error("Buggy Data file:\n %s" % data_lfn)
      gLogger.notice("Setting nsb=-1")
      res = fcc.setMetadata(data_lfn,{'nsb': -1})
      if not res['OK']:
        return res

    code = os.system("rm -Rf " + tardir)
    if code!=0:
      gLogger.error("Unable to remove tar directory")
      DIRAC.exit(-1)
    code = os.system("rm " + file_name)
    if code!=0:
     gLogger.error("Unable to remove Log file:\n %s" % file_name)
     DIRAC.exit(-1)

  DIRAC.exit()

if __name__ == '__main__':
  main()



"""
  Simple Wrapper on the Job class to handle Prod3 MC
  piping corsika into simtels, running Prod3 divergent pointing simulations
			JB,LA March 2018
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections
# DIRAC imports
import DIRAC
from DIRAC.Interfaces.API.Job import Job


class Prod3MCPipeDivergentJob(Job):
  """ Job extension class for Prod3 MC simulations,
  takes care of running corsika piped into simtel
  one output file is created
  """

  def __init__(self, cpuTime=259200):
    """ Constructor

    Keyword arguments:
    cpuTime -- max cpu time allowed for the job
    """
    Job.__init__(self)
    self.setCPUTime(cpuTime)
    # defaults
    self.setName('Prod3MC_Generation')
    self.package = 'corsika_simhessarray'
    self.program_category = 'tel_sim'
    self.version = '2017-04-19-div'
    self.configuration_id = 3
    self.output_data_level = 0
    self.nShower = 100
    self.start_run_number = '0'
    self.run_number = '10'
    self.cta_site = 'Paranal'
    self.cta_site_tag = ''
    self.array_layout = 'Baseline'
    self.particle = 'gamma'
    self.pointing_dir = 'South'
    self.zenith_angle = 20.
    self.div_cfg_id = 0
    self.inputpath = 'Data/sim_telarray/cta-prod3-demo/0.0deg'
    self.N_output_files = 1
    self.basepath = '/vo.cta.in2p3.fr/MC/PROD3/'
    self.catalogs = json.dumps(['DIRACFileCatalog', 'TSCatalog'])
    self.metadata = collections.OrderedDict()
    self.metadata_field = dict()
    self.filemetadata = dict()

  def setArrayLayout(self, layout):
    """ Set the array layout type

    Parameters:
    layout -- a string for the array layout, hex or square
    """
    if layout in ['Baseline']:
      DIRAC.gLogger.info('Set Simtel layout to: %s' % layout)
      self.array_layout = layout
    else:
      DIRAC.gLogger.error('Unknown layout: : %s' % layout)
      DIRAC.exit(-1)

  def setSite(self, site):
    """ Set the site to simulate

    Parameters:
    site -- a string for the site name (LaPalma)
    """
    self.cta_site = site
    if self.cta_site not in ['LaPalma']:
      DIRAC.gLogger.error('Divergent setup available only for LaPalma')
      DIRAC.exit(-1)

  def setParticle(self, particle):
    """ Set the corsika primary particle

    Parameters:
    particle -- a string for the particle type/name
    """
    if particle in ['gamma', 'gamma-diffuse', 'electron',
                    'proton', 'helium']:
      DIRAC.gLogger.info('Set Corsika particle to: %s' % particle)
      self.particle = particle
    else:
      DIRAC.gLogger.error('Corsika does not know particle type: %s'
                          % particle)
      DIRAC.exit(-1)

  def setPointingDir(self, pointing):
    """ Set the pointing direction, North or South

    Parameters:
    pointing -- a string for the pointing direction
    """
    if pointing in ['North', 'South', 'East', 'West']:
      DIRAC.gLogger.info('Set Pointing dir to: %s' % pointing)
      self.pointing_dir = pointing
    else:
      DIRAC.gLogger.error('Unknown pointing direction: %s' % pointing)
      DIRAC.exit(-1)

  def setMetaData(self):
    """ define output meta data based on default and current
    attribute values

    The order of the metadata dictionary is important,
    since it's used to build the directory structure
    """
    # defin the meta data dictionnary
    self.metadata['array_layout'] = self.array_layout
    self.metadata['site'] = self.cta_site
    self.metadata['particle'] = self.particle
    if self.pointing_dir == 'North':
      self.metadata['phiP'] = 180
    if self.pointing_dir == 'South':
      self.metadata['phiP'] = 0
    self.metadata['thetaP'] = float(self.zenith_angle)
    self.metadata['tel_sim_prog'] = 'simtel'
    self.metadata['tel_sim_prog_version'] = self.version
    self.metadata['data_level'] = self.output_data_level
    self.metadata['configuration_id'] = self.configuration_id

    # define also the metadat field
    self.metadata_field = {'array_layout': 'VARCHAR(128)',
                           'site': 'VARCHAR(128)',
                           'particle': 'VARCHAR(128)', 'phiP': 'float',
                           'thetaP': 'float',
                           self.program_category + '_prog': 'VARCHAR(128)',
                           self.program_category + '_prog_version': 'VARCHAR(128)',
                           'data_level': 'int', 'configuration_id': 'int'}

  def setupWorkflow(self, debug=False, register=True):
    """ Setup job workflow by defining the sequence of all executables
        All parameters shall have been defined before that method is called
    """
    # step 1 - debug only
    iStep = 1
    if debug:
      lsStep = self.setExecutable('/bin/ls -alhtr',
                                  logFile='LS_Init_Log.txt')
      lsStep['Value']['name'] = 'Step%i_LS_Init' % iStep
      lsStep['Value']['descr_short'] = 'list files in working directory'
      iStep += 1

    # step 2
    swStep = self.setExecutable('cta-prod3-setupsw',
                                arguments='%s %s' % (self.package,
                                                     self.version),
                                logFile='SetupSoftware_Log.txt')
    swStep['Value']['name'] = 'Step%i_SetupSoftware' % iStep
    swStep['Value']['descr_short'] = 'Setup software'
    iStep += 1

    # step 3
    if self.cta_site == 'Paranal':
      prod_script = './dirac_prod3_paranal_divergent'
      DIRAC.gLogger.error('No setup available for Paranal yet')
      DIRAC.exit(-1)
    elif self.cta_site == 'LaPalma':
      prod_script = './dirac_prod3_lapalma_divergent'
    else:
      DIRAC.gLogger.error('Unknown site: %s' % self.cta_site)
      DIRAC.gLogger.error('No shell script associated')
      DIRAC.exit(-1)

    csStep = self.setExecutable(prod_script,
                                arguments='--start_run %s --run %s %s %s %s %s' %
                                (self.start_run_number, self.run_number,
                                 self.cta_site, self.particle,
                                 self.pointing_dir, self.zenith_angle),
                                logFile='CorsikaSimtel_Log.txt')
    csStep['Value']['name'] = 'Step%i_CorsikaSimtel' % iStep
    csStep['Value']['descr_short'] = 'Run Corsika piped into simtel'
    iStep += 1

    # step 4 verify merged data
    mgvStep = self.setExecutable('cta-prod-verifysteps',
                                 arguments="generic %0d 1000 '%s/Data/*.simtel.gz'" %
                                 (self.N_output_files, self.inputpath),
                                 logFile='Verify_Simtel_Log.txt')
    mgvStep['Value']['name'] = 'Step%i_VerifySimtel' % iStep
    mgvStep['Value']['descr_short'] = 'Verify simtel files'
    iStep += 1

    # step 5 - debug only
    if debug:
      lsStep = self.setExecutable('/usr/bin/find',
                                  arguments='.',
                                  logFile='LS_End_Log.txt')
      lsStep['Value']['name'] = 'Step%i_LS_End' % iStep
      lsStep['Value']['descr_short'] = 'list files in working directory'
      iStep += 1

    # step 6
    if register:
      # call setMetaData that defines self.metadata and self.metadata_field
      self.setMetaData()
      mdjson = json.dumps(self.metadata)
      mdfieldjson = json.dumps(self.metadata_field)

      filemetadata = {'runNumber': self.run_number}
      file_md_json = json.dumps(filemetadata)

      scripts = '../CTADIRAC/Core/scripts'
      dmStep = self.setExecutable(scripts + '/cta-prod3-managedata.py',
                                  arguments="'%s' '%s' '%s' %s %s %s '%s'" %
                                  (mdjson, mdfieldjson, file_md_json,
                                   self.inputpath,
                                   self.basepath, self.start_run_number,
                                   self.catalogs),
                                  logFile='DataManagement_Log.txt')
      dmStep['Value']['name'] = 'Step%i_DataManagement' % iStep
      dmStep['Value']['descr_short'] = 'Save files to SE and register them in DFC'
      iStep += 1

    # Number of showers and divergent configuratino id are passed
    # via an environment variable
    self.setExecutionEnv({'NSHOW': '%s' % self.nShower,
                          'DIV_CFG_ID': '%s' % self.div_cfg_id})



"""
  Simple Wrapper on the Job class to handle Prod3 MC
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections
# DIRAC imports
import DIRAC
from DIRAC.Interfaces.API.Job import Job


class Prod3MCJob(Job):
  """ Job extension class for Prod3 MC simulations,
      takes care of running corsika, 31 simtels and merging
      into 5 data files and 3 tar ball for log files.
  """

  def __init__(self, cpuTime=432000):
    """ Constructor

    Keyword arguments:
    cpuTime -- max cpu time allowed for the job
    """
    Job.__init__(self)
    self.setCPUTime(cpuTime)
    # defaults
    self.setName('Prod3MC_Generation')
    self.package = 'corsika_simhessarray'
    self.version = '2015-07-21'
    self.nShower = 5
    self.start_run_number = '0'
    self.run_number = '10'
    self.array_layout = 'hex'
    self.template_tag = '6'
    self.cta_site = 'Paranal'
    self.particle = 'gamma'
    self.pointing_dir = 'South'
    self.zenith_angle = 20.
    self.no_sct = True
    self.inputpath = 'Data/sim_telarray/cta-prod3/0.0deg'
    self.basepath = '/vo.cta.in2p3.fr/MC/PROD3/scratch'

  def setPackage(self, package):
    """ Set package name : e.g. 'corsika_simhessarray'

    Parameters:
    package -- corsika_simhessarray
    """
    self.package = package

  def setVersion(self, version):
    """ Set software version number : e.g. 2015-07-21

    Parameters:
    version -- corsika+simtel package version number
    """
    self.version = version

  def setNShower(self, nshow):
    """ Set the number of corsika showers,
        5 is enough for testing
        20000 in production usually.

    Parameters:
    nshow -- number of corsika primary showers to generate
    """
    self.nShower = nshow

  def setRunNumber(self, runNb):
    """ Set the corsika run number, passed as a string
        because may be a TS parameter

    Parameters:
    runNb -- run number as a string, used as a corsika seed
    """
    self.run_number = runNb

  def setStartRunNumber(self, startrunNb):
    """ Set the corsika start run number (to be added to the run_number), passed as a string
        because may be a TS parameter

    Parameters:
    startrunNb -- to be added to the run number
    """
    self.start_run_number = startrunNb

  def setArrayLayout(self, layout):
    """ Set the array layout type

    Parameters:
    layout -- a string for the array layout, hex or square
    """
    if layout in ['hex', 'square']:
      DIRAC.gLogger.info('Set Simtel layout to: %s' % layout)
      self.array_layout = layout
    else:
      DIRAC.gLogger.error('Unknown layout: : %s' % layout)
      DIRAC.exit(-1)

  def setSite(self, site):
    """ Set the site to simulate

    Parameters:
    site -- a string for the site name (Paranal)
    """
    self.cta_site = site

  def setParticle(self, particle):
    """ Set the corsika primary particle

    Parameters:
    particle -- a string for the particle type/name
    """
    if particle in ['gamma', 'gamma-diffuse', 'electron', 'proton', 'helium']:
      DIRAC.gLogger.info('Set Corsika particle to: %s' % particle)
      self.particle = particle
    else:
      DIRAC.gLogger.error('Corsika does not know particle type: %s' % particle)
      DIRAC.exit(-1)

  def setPointingDir(self, pointing):
    """ Set the pointing direction, North or South

    Parameters:
    pointing -- a string for the pointing direction
    """
    if pointing in ['North', 'South', 'East', 'West']:
      DIRAC.gLogger.info('Set Pointing dir to: %s' % pointing)
      self.pointing_dir = pointing
    else:
      DIRAC.gLogger.error('Unknown pointing direction: %s' % pointing)
      DIRAC.exit(-1)

  def setZenithAngle(self, zenith):
    """ Set the pointing direction, North or South

    Parameters:
    zenith -- a string for the zenith angle
    """
    self.zenith_angle = zenith

  def setNoSCTFlag(self, no_sct):
    """ Set the no sct flag to true or false

    Parameters:
    no_sct -- a bool set to true if you do not want to simulate SCTs
    """
    self.no_sct = no_sct

  def setupWorkflow(self, debug=False):
    """ Setup job workflow by defining the sequence of all executables
        All parameters shall have been defined before that method is called.
    """
    # step 1 -- to be removed -- debug only
    iStep = 1
    if debug:
      lsStep = self.setExecutable('/bin/ls -alhtr', logFile='LS_Init_Log.txt')
      lsStep['Value']['name'] = 'Step%i_LS_Init' % iStep
      lsStep['Value']['descr_short'] = 'list files in working directory'
      iStep += 1

    # step 2
    swStep = self.setExecutable('cta-prod3-setupsw',
                                arguments='%s %s' % (self.package, self.version),
                                logFile='SetupSoftware_Log.txt')
    swStep['Value']['name'] = 'Step%i_SetupSoftware' % iStep
    swStep['Value']['descr_short'] = 'Setup software'
    iStep += 1

    # step 3
    csStep = self.setExecutable('./dirac_prod3_corsika',
                                arguments='--start_run %s --run %s %s-%s %s %s %s %s' %
                                (self.start_run_number, self.run_number,
                                 self.array_layout, self.template_tag, self.cta_site,
                                 self.particle, self.pointing_dir, self.zenith_angle),
                                logFile='Corsika_Log.txt')
    csStep['Value']['name'] = 'Step%i_Corsika' % iStep
    csStep['Value']['descr_short'] = 'Run Corsika with 800+ telescopes'
    iStep += 1

    # step 4
    csvStep = self.setExecutable('cta-prod-verifysteps',
                                 arguments='corsika 6 100',
                                 logFile='Verify_Corsika_Log.txt')
    csvStep['Value']['name'] = 'Step%i_VerifyCorsika' % iStep
    csvStep['Value']['descr_short'] = 'Verify the Corsika run'
    iStep += 1

    # step 5
    simtel_args = '--start_run %s --run %s --layout %s' % (self.start_run_number, self.run_number, self.array_layout)
    if self.no_sct:
      simtel_args += ' --no-sct 1'
    stStep = self.setExecutable('./dirac_prod3_simtel',
                                arguments=simtel_args,
                                logFile='Simtels_Log.txt')
    stStep['Value']['name'] = 'Step%i_Simtel' % iStep
    stStep['Value']['descr_short'] = 'Run 31 simtel_array configuration sequentially'
    iStep += 1

    # step 6
    verif_simtel_args = 'simtel'
    if self.no_sct:
      verif_simtel_args += ' 26 100'
    else:
      verif_simtel_args += ' 31 100'
    stvStep = self.setExecutable('cta-prod-verifysteps',
                                 arguments=verif_simtel_args,
                                 logFile='Verify_Simtel_Log.txt')
    stvStep['Value']['name'] = 'Step%i_VerifySimtel' % iStep
    stvStep['Value']['descr_short'] = 'Verify the 26 or 31 Simtel runs'
    iStep += 1

    # step 7
    cleanStep = self.setExecutable('cta-prod3-cleandata',
                                   arguments="%s %s" % ('Data/corsika', '*/*.corsika.gz'),
                                   logFile='CleanData_Log.txt')
    cleanStep['Value']['name'] = 'Step%i_CleanData' % iStep
    cleanStep['Value']['descr_short'] = 'Remove corsika files'
    iStep += 1

    # step 8
    mgStep = self.setExecutable('./dirac_prod3_merge',
                                arguments=simtel_args,
                                logFile='Merging_Log.txt')
    mgStep['Value']['name'] = 'Step%i_Merging' % iStep
    mgStep['Value']['descr_short'] = 'Merge 31 simtel output into 5 data files and 3 tar balls for log files'
    iStep += 1

    # step 9
    verif_merging_args = 'merging'
    if self.no_sct:
      verif_merging_args += ' 5 5000'
    else:
      verif_merging_args += ' 10 5000'
    mgvStep = self.setExecutable('cta-prod-verifysteps',
                                 arguments=verif_merging_args,
                                 logFile='Verify_Merging_Log.txt')
    mgvStep['Value']['name'] = 'Step%i_VerifyMerging' % iStep
    mgvStep['Value']['descr_short'] = 'Verify the merging of Simtel files'
    iStep += 1

    # step 10  - to be removed - debug only
    if debug:
      lsStep = self.setExecutable('/bin/ls -alhtr Data/sim_telarray/cta-prod3/*/*', logFile='LS_End_Log.txt')
      lsStep['Value']['name'] = 'Step%i_LS_End' % iStep
      lsStep['Value']['descr_short'] = 'list files in working directory and sub-directory'
      iStep += 1

    # step 11
    # ## the order of the metadata dictionary is important, since it's used to build the directory structure
    metadata = collections.OrderedDict()
    metadata['array_layout'] = self.array_layout
    metadata['site'] = self.cta_site
    metadata['particle'] = self.particle
    if self.pointing_dir == 'North':
      metadata['phiP'] = 180
    if self.pointing_dir == 'South':
      metadata['phiP'] = 0
    metadata['thetaP'] = float(self.zenith_angle)
    # metadata['process_program'] = 'simtel' + '_' + self.version
    metadata['tel_sim_prog'] = 'simtel'
    metadata['tel_sim_prog_version'] = self.version

    mdjson = json.dumps(metadata)

    metadatafield = {
        'array_layout': 'VARCHAR(128)',
        'site': 'VARCHAR(128)',
        'particle': 'VARCHAR(128)',
        'phiP': 'float',
        'thetaP': 'float',
        'tel_sim_prog': 'VARCHAR(128)',
        'tel_sim_prog_version': 'VARCHAR(128)'}

    mdfieldjson = json.dumps(metadatafield)

    filemetadata = {'runNumber': self.run_number}

    fmdjson = json.dumps(filemetadata)

    # Temporary fix: since the deployed script does not have the correct format for arguments
    # dmStep = self.setExecutable( '$DIRACROOT/scripts/cta-prod3-managedata',
    dmStep = self.setExecutable(
        'cta-prod3-managedata',
        arguments="'%s' '%s' '%s' %s %s %s" %
        (mdjson,
         mdfieldjson,
         fmdjson,
         self.inputpath,
         self.basepath,
         self.start_run_number),
        logFile='DataManagement_Log.txt')
    dmStep['Value']['name'] = 'Step%i_DataManagement' % iStep
    dmStep['Value']['descr_short'] = 'Save files to SE and register them in DFC'
    iStep += 1

    # Do not merge SCT as this requires too much memory
    # number of showers is passed via an environment variable
    self.setExecutionEnv({'NSHOW': '%s' % self.nShower,
                          'NO_SCT_MERGE': 1})



#!/usr/bin/env python

__RCSID__ = "$Id$"

# generic imports
from multiprocessing import Pool

# DIRAC imports
from DIRAC import gLogger
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script

Script.setUsageMessage("""
Bulk removal of a list of files
Usage:
   cta-prod-remove-file [options] <ascii file with lfn list>
""")

Script.parseCommandLine(ignoreErrors=True)

from DIRAC.Resources.Catalog.FileCatalog import FileCatalog
fc = FileCatalog()

def removeFile(lfn):
  res = fc.removeFile(lfn)
  if not res['OK']:
    gLogger.error('Error removing file', lfn)
    return res['Message']

@Script()
def main():
  args = Script.getPositionalArgs()
  if len(args) > 0:
    infile = args[0]
  else:
    Script.showHelp()

  infileList = read_inputs_from_file(infile)
  p = Pool(10)
  p.map(removeFile, infileList)

if __name__ == '__main__':
  main()



#!/usr/bin/env python
"""
Show the progress of Moving transformations associated to one or more datasets

Warning: Moving transformations are expected to follow the naming convention:
<Move>_<MCCampaign>_<datasetName>
Transformations not following this convention can be inspected directly using the Transformation Web Monitor or the transformation CLI

Usage:
   cta-prod-move-dataset-status <datasetName (may contain wild cards) or ascii file with a list of datasets> <MCCampaign> <target SE>

Example:
   cta-prod-move-dataset-status Paranal_gamma_North_20deg_HB9 PROD3 CC-IN2P3-Tape
"""

__RCSID__ = "$Id$"

import os

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient

def get_list_of_datasets(tag=''):
  fc = FileCatalogClient()
  dataset_tag = '*%s*' % tag
  res = fc.getDatasets(dataset_tag)
  if not res['OK']:
    gLogger.error(res['Message'])
    DIRAC.exit(-1)
  dataset_dict = res['Value']
  dataset_list = list()
  for dataset_name in dataset_dict['Successful'][dataset_tag]:
    dataset_list.append(dataset_name)
  return dataset_list

def get_dataset_info(dataset_name):
  fc = FileCatalogClient()
  res = fc.getDatasets(dataset_name)
  if not res['OK']:
    gLogger.error("Failed to get datasets")
    DIRAC.exit(-1)
  dataset_dict = res['Value']
  res = dataset_dict['Successful'][dataset_name][dataset_name]
  number_of_files = res['NumberOfFiles']
  meta_query = res['MetaQuery']
  total_size = res['TotalSize']
  return (dataset_name, number_of_files, total_size, meta_query)

def get_transformation_info(dataset_name, MCCampaign, targetSE):

  transClient = TransformationClient()

  paramShowNames = [
      "TransformationID",
      "TransformationName",
      "Type",
      "Status",
      "Files_Total",
      "Files_PercentProcessed",
      "Files_Processed",
      "Files_Unused",
      "Jobs_TotalCreated",
      "Jobs_Waiting",
      "Jobs_Running",
      "Jobs_Done",
      "Jobs_Failed",
      "Jobs_Stalled",
  ]

  trans_name = ("Move_%s_%s_%s" % (MCCampaign, targetSE, dataset_name))
  res = transClient.getTransformationParameters(trans_name, 'TransformationID')

  if not res['OK']:
    gLogger.error("Warning: failed to get transformation %s" % (trans_name))
    return "_"

  transID = res['Value']
  res = transClient.getTransformationSummaryWeb({"TransformationID": transID}, [], 0, 1)

  if not res["OK"]:
      DIRAC.gLogger.error(res["Message"])
      DIRAC.exit(-1)

  if res["Value"]["TotalRecords"] > 0:
    paramNames = res["Value"]["ParameterNames"]
    for paramValues in res["Value"]["Records"]:
      paramShowValues = map(lambda pname: paramValues[paramNames.index(pname)], paramShowNames)
      showDict = dict(zip(paramShowNames, paramShowValues))
      files_PercentProcessed = showDict["Files_PercentProcessed"]

  return files_PercentProcessed

#########################################################
@Script()
def main():

  _, argss = Script.parseCommandLine(ignoreErrors=True)

  if len(argss) != 3:
    Script.showHelp()

  if os.path.isfile(argss[0]):
    gLogger.notice('\nReading datasets from input file: %s\n' % argss[0])
    dataset_list = read_inputs_from_file(argss[0])
  else:
    dataset_name = argss[0]
    dataset_list = list()
    if dataset_name.find('*') > 0:
      dataset_list = get_list_of_datasets(dataset_name)
      if len(dataset_list) == 0:
        gLogger.notice("No dataset found")
        DIRAC.exit()
    else:
      dataset_list.append(dataset_name)

  MCCampaign =  argss[1]
  targetSE =  argss[2]

  values = []
  for dataset_name in dataset_list:
    name, n_files, size, mq = get_dataset_info(dataset_name)
    # # convert total size in TB
    size_TB = size / 1e12
    # get transformation info
    files_PercentProcessed = get_transformation_info(dataset_name, MCCampaign, targetSE)
    values.append((name, n_files, float("{:.2f}".format(size_TB)), files_PercentProcessed))

  # print table
  gLogger.notice('\n|_. Name |_. N files |_. Size(TB) |_. Files Processed (%)| ')
  for value in values:
    gLogger.notice("|%s|" % "|".join(map(str,value)))

  DIRAC.exit()

if __name__ == '__main__':
  main()



"""
  Simple Wrapper on the Job class to handle EvnDisp stereo reconstruction
  runs mscw_energy for DL1 to DL2 using look up tables
  Specialized for Reference setup simulation with the Baseline layout
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections
# DIRAC imports
from DIRAC.Interfaces.API.Job import Job
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient

class EvnDisp3MSCWRefJob( Job ) :
  """ Job extension class for EvnDisp Analysis,
      takes care of running mscw_energy.
  """

  def __init__( self, cpuTime = 432000 ):
    """ Constructor

    Keyword arguments:
    cpuTime -- max cpu time allowed for the job
    """
    Job.__init__( self )
    self.setCPUTime( cpuTime )
    # defaults
    self.setName('Evndisplay_Reco')
    self.package='evndisplay'
    self.program_category='reconstruction'
    self.version = 'prod3b_d20170602' # or later
    self.configuration_id = 0
    self.output_data_level=2
    self.prefix = 'CTA.prod3Nb'
    self.layout = 'Baseline'
    self.pointing = '180'
    self.table_file='tables_CTA-prod3b-LaPalma-NNq05-NN-ID0_0deg-d20160925m4-Nb.3AL4-BN15.root'
    self.disp_subdir_name = 'BDTdisp.Nb.3AL4-BN15.T1'
    self.recid  = '0,1,2' # 0 = all teltescopes, 1 = LST only, 2 = MST only
    self.basepath = '/vo.cta.in2p3.fr/MC/PROD3/'
    self.fcc = FileCatalogClient()
    self.metadata = collections.OrderedDict()
    self.filemetadata = {}
    self.catalogs = json.dumps(['DIRACFileCatalog','TSCatalog'])

  def setTSTaskId(self, taskid):
    """ Set TS task Id, dynamically resolved at job run time

    Parameters:
    taskid -- an int
    """
    self.ts_task_id=taskid

  def setPackage(self, package):
    """ Set package name : e.g. 'evndisplay'

    Parameters:
    package -- evndisplay
    """
    self.package=package

  def setVersion(self, version):
    """ Set software version number : e.g. v500-prod3v1

    Parameters:
    version -- evndisplay package version number
    """
    self.version=version

  def setPrefix( self, prefix ):
    """ Set prefix for layout name

    Parameters:
    prefix -- prefix for layout names
    """
    self.prefix = prefix

  def setLayout( self, layout ):
    """ Set the layout list

    Parameters:
    layout -- the layout "Baseline"
    """
    self.layout = layout

  def setPointing( self, angle ):
    """ Set the pointing direction for reconstruction
    @todo could be made North or South ?

    Parameters:
    pointing -- an angle in degrees 0 or 180
    """
    self.pointing = angle

  def setTableFile( self, table ):
    """ Set the table file name used for reconstruction
    This is the look up table for direction and energy
    File will be look for in directory "$CTA_EVNDISP_AUX_DIR/Tables/"

    Parameters:
    table -- lookup table file name
    """
    self.table_file = table

  def setDispSubDir( self, dispdir ):
    """ Set the BDT directory name

    Parameters:
    dispdir -- sub-directory name
    """
    self.disp_subdir_name = dispdir

  def setRecId( self, recid ):
    """ Set the list of telescopes sub-layout to reconstruction
    For North site (La Palma):
    + 0 = full array, LSTs+MSTs
    + 1 = 4 LSTs only
    + 2 = 15 MSTs only

    Parameters:
    recid -- a coma separated list of ints (0,1,2)
    """
    self.recid = recid

  def setEvnDispMD( self,  simtelMD ):
    """ Set evndisplay meta data

    Parameters:
    simtelMD -- metadata dictionary as for simtel
    """
    # # Set evndisp directory meta data
    self.metadata['array_layout'] = simtelMD['array_layout']
    self.metadata['site'] = simtelMD['site']
    self.metadata['particle'] = simtelMD['particle']
    self.metadata['phiP'] = simtelMD['phiP']['=']
    self.metadata['thetaP'] = simtelMD['thetaP']['=']
    self.metadata[self.program_category+'_prog'] = 'evndisp'
    self.metadata[self.program_category+'_prog_version'] = self.version
    self.metadata['data_level'] = self.output_data_level
    self.metadata['configuration_id'] = self.configuration_id
    

  def setupWorkflow(self, debug=False):
    """ Setup job workflow by defining the sequence of all executables
        All parameters shall have been defined before that method is called.
    """

    # step 1 -- to be removed -- debug only
    iStep = 1
    if debug:
        lsStep = self.setExecutable( '/bin/ls -alhtr', logFile = 'LS_Init_Log.txt' )
        lsStep['Value']['name']='Step%i_LS_Init'%iStep
        lsStep['Value']['descr_short']='list files in working directory'
        iStep+=1

    # step 2
    swStep = self.setExecutable( 'cta-prod3-setupsw',
                              arguments='%s %s'% (self.package, self.version),\
                              logFile='SetupSoftware_Log.txt')
    swStep['Value']['name'] = 'Step%i_SetupSoftware' % iStep
    swStep['Value']['descr_short'] = 'Setup software'
    iStep+=1

    # step 3
    # arguments are nbFiles=0 (not used) and fileSize=100kB
    eivStep = self.setExecutable( 'cta-prod-verifysteps', \
                              arguments = 'analysisinputs 0 100', \
                              logFile = 'Verify_EvnDispInputs_Log.txt' )
    eivStep['Value']['name'] = 'Step%i_VerifyEvnDispInputs' % iStep
    eivStep['Value']['descr_short'] = 'Verify EvnDisp Inputs'
    iStep += 1

    # step 4
    evStep = self.setExecutable( './dirac_prod3_evndisp_mscw_ref', \
        arguments = "--prefix %s --layout '%s' --pointing %s --table_file %s\
                     --disp_subdir_name %s --recid '%s' --taskid %s" %\
                    ( self.prefix, self.layout, self.pointing, self.table_file,\
                      self.disp_subdir_name, self.recid, self.ts_task_id),\
        logFile = 'EvnDisp_MSCW_Log.txt' )
    evStep['Value']['name'] = 'Step%i_EvnDisplay_MSMCW' % iStep
    evStep['Value']['descr_short'] = 'Run EvnDisplay mscw_energy'
    iStep += 1

    # step 5
    # ## the order of the metadata dictionary is important, since it's used to build the directory structure
    mdjson = json.dumps( self.metadata )
    metadatafield = {'array_layout':'VARCHAR(128)', 'site':'VARCHAR(128)',
                     'particle':'VARCHAR(128)', 'phiP':'float',
		          'thetaP': 'float',
		          self.program_category+'_prog':'VARCHAR(128)',
		          self.program_category+'_prog_version':'VARCHAR(128)',
                     'data_level': 'int', 'configuration_id': 'int'}
    mdfieldjson = json.dumps(metadatafield)

    # register Data
    outputpattern = './Data/*DL%01d.root'%self.output_data_level
    filemetadata = {'data_level': self.output_data_level,
                    'configuration_id':self.configuration_id}
    file_md_json = json.dumps(filemetadata)
    dmStep = self.setExecutable('cta-analysis-managedata',
                              arguments = "'%s' '%s' '%s' %s '%s' %s %s '%s'" %\
                              (mdjson, mdfieldjson, file_md_json, self.basepath,
                               outputpattern, self.package, self.program_category, self.catalogs),
                              logFile = 'DataManagement_Log.txt')
    dmStep['Value']['name'] = 'Step%i_DataManagement' % iStep
    dmStep['Value']['descr_short'] = 'Save data files to SE and register them in DFC'
    iStep += 1

    # register Log
    outputpattern = './*.logs.tgz'
    filemetadata = {}
    file_md_json = json.dumps(filemetadata)
    dmStep = self.setExecutable('cta-analysis-managedata',
                              arguments = "'%s' '%s' '%s' %s '%s' %s %s '%s' Log" % \
                              (mdjson, mdfieldjson, file_md_json, self.basepath,
                               outputpattern, self.package, self.program_category, self.catalogs),
                              logFile = 'Log_DataManagement_Log.txt')
    dmStep['Value']['name'] = 'Step%i_Log_DataManagement' % iStep
    dmStep['Value']['descr_short'] = 'Save log files to SE and register them in DFC'
    iStep += 1

    # step 6 -- to be removed -- debug only
    if debug:
        lsStep = self.setExecutable('/bin/ls',
                                    arguments = " -alhtr; /bin/ls -lahrt ./Data",
                                    logFile = 'LS_End_Log.txt')
        lsStep['Value']['name']='Step%i_LS_End'%iStep
        lsStep['Value']['descr_short']='list files in working directory and in Data directory'
        iStep+=1



"""
  Job wrapper class to handle Prod4 MC
  Corsika simulations of the SSTs
			JB September 2018
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections
# DIRAC imports
import DIRAC
from DIRAC.Interfaces.API.Job import Job
from CTADIRAC.Core.Utilities.tool_box import DATA_LEVEL_METADATA_ID


class Prod4CorsikaSSTJob(Job):
    """ Job extension class for Prod4 MC simulations,
      takes care of running corsika for the SST only template
    """
    def __init__(self, cpu_time=129600):
        """ Constructor

        Keyword arguments:
        cpuTime -- max cpu time allowed for the job
        """
        Job.__init__(self)
        self.setCPUTime(cpu_time)
        self.setName('Prod4_MC_Generation')
        self.setType('MCSimulation')
        self.package = 'corsika_simhessarray'
        self.program_category = 'airshower_sim'
        self.prog_name = 'corsika'
        self.version = '2018-09-19'
        self.configuration_id = 4
        self.output_data_level = DATA_LEVEL_METADATA_ID['MC0']
        self.start_run_number = '0'
        self.run_number = '10'
        self.n_shower = 100
        self.cta_site = 'Paranal'
        self.particle = 'gamma'
        self.pointing_dir = 'South'
        self.zenith_angle = 20.
        self.output_pattern = 'Data/corsika/run*/*corsika.zst'
        self.n_output_files = 1
        self.output_file_size = 1000  # kb
        self.base_path = '/vo.cta.in2p3.fr/MC/PROD4/'
        self.catalogs = json.dumps(['DIRACFileCatalog', 'TSCatalog'])
        self.metadata = collections.OrderedDict()

    def set_site(self, site):
        """ Set the site to simulate

        Parameters:
        site -- a string for the site name (LaPalma)
        """
        if site in ['Paranal', 'LaPalma']:
            DIRAC.gLogger.info('Set Corsika site to: %s' % site)
            self.cta_site = site
        else:
            DIRAC.gLogger.error('Site is unknown: %s' % site)
            DIRAC.exit(-1)

    def set_particle(self, particle):
        """ Set the corsika primary particle

        Parameters:
        particle -- a string for the particle type/name
        """
        if particle in ['gamma', 'gamma-diffuse', 'electron', 'proton', 'helium']:
            DIRAC.gLogger.info('Set Corsika particle to: %s' % particle)
            self.particle = particle
        else:
            DIRAC.gLogger.error('Corsika does not know particle type: %s' % particle)
            DIRAC.exit(-1)

    def set_pointing_dir(self, pointing):
        """ Set the pointing direction, North or South

        Parameters:
        pointing -- a string for the pointing direction
        """
        if pointing in ['North', 'South', 'East', 'West']:
            DIRAC.gLogger.info('Set Pointing dir to: %s' % pointing)
            self.pointing_dir = pointing
        else:
            DIRAC.gLogger.error('Unknown pointing direction: %s' % pointing)
            DIRAC.exit(-1)

    def set_meta_data(self):
        """ define the common meta data of the application
        """
        # The order of the metadata dictionary is important,
        # since it's used to build the directory structure
        self.metadata['array_layout'] = 'Baseline-SST-only'
        self.metadata['site'] = self.cta_site
        self.metadata['particle'] = self.particle
        # air shower simulation means North=0 and South=180
        if self.pointing_dir == 'North':
            self.metadata['phiP'] = 0
        if self.pointing_dir == 'South':
            self.metadata['phiP'] = 180
        self.metadata['thetaP'] = float(self.zenith_angle)
        self.metadata[self.program_category + '_prog'] = self.prog_name
        self.metadata[self.program_category + '_prog_version'] = self.version
        self.metadata['data_level'] = self.output_data_level
        self.metadata['configuration_id'] = self.configuration_id

    def setupWorkflow(self, debug=False):
        """ Setup job workflow by defining the sequence of all executables
            All parameters shall have been defined before that method is called.
        """
        # Step 1 - debug only
        i_step = 1

        # Step 2 - setup software
        sw_step = self.setExecutable('cta-prod3-setupsw',
                                     arguments='%s %s' % (self.package, self.version),
                                     logFile='SetupSoftware_Log.txt')
        sw_step['Value']['name'] = 'Step%i_SetupSoftware' % i_step
        sw_step['Value']['descr_short'] = 'Setup software'
        i_step += 1

        if debug:
            ls_step = self.setExecutable('/bin/ls -alhtr', logFile='LS_Init_Log.txt')
            ls_step['Value']['name'] = 'Step%i_LS_Init' % i_step
            ls_step['Value']['descr_short'] = 'list files in working directory'
            i_step += 1

        # Step 3 - run corsika
        if self.cta_site == 'Paranal':
            prod_script = './dirac_prod4_sst-only_run'
        else:
            DIRAC.gLogger.error('Site not supported: %s' % self.cta_site)
            DIRAC.gLogger.error('No shell script associated')
            DIRAC.exit(-1)

        cs_step = self.setExecutable(prod_script,
                                     arguments='--without-multipipe --start_run %s \
                                               --run %s %s %s %s %s' %
                                     (self.start_run_number, self.run_number,
                                      self.cta_site, self.particle,
                                      self.pointing_dir, self.zenith_angle),
                                     logFile='Corsika_Log.txt')
        cs_step['Value']['name'] = 'Step%i_Corsika' % i_step
        cs_step['Value']['descr_short'] = 'Run Corsika only'
        i_step += 1

        # Step 4 - verify size of corsika output
        csv_step = self.setExecutable('cta-prod-verifysteps',
                                      arguments='generic %d %d %s' %
                                      (self.n_output_files, self.output_file_size,
                                       self.output_pattern),
                                      logFile='Verify_Corsika_Log.txt')
        csv_step['Value']['name'] = 'Step%i_VerifyCorsika' % i_step
        csv_step['Value']['descr_short'] = 'Verify the Corsika run'
        i_step += 1

        # Step 5 - define meta data, upload file on SE and register in catalogs
        self.set_meta_data()
        md_json = json.dumps(self.metadata)

        meta_data_field = {'array_layout': 'VARCHAR(128)', 'site': 'VARCHAR(128)',
                           'particle': 'VARCHAR(128)',
                           'phiP': 'float', 'thetaP': 'float',
                           self.program_category + '_prog': 'VARCHAR(128)',
                           self.program_category + '_prog_version': 'VARCHAR(128)',
                           'data_level': 'int', 'configuration_id': 'int'}
        md_field_json = json.dumps(meta_data_field)

        # Upload and register data
        file_meta_data = {}
        file_md_json = json.dumps(file_meta_data)

        scripts = '../CTADIRAC/Core/scripts/'
        dm_step = self.setExecutable(scripts + 'cta-prod-managedata.py',
                                     arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data" %
                                     (md_json, md_field_json, file_md_json,
                                      self.base_path, self.output_pattern, self.package,
                                      self.program_category, self.catalogs),
                                     logFile='DataManagement_Log.txt')
        dm_step['Value']['name'] = 'Step%s_DataManagement' % i_step
        dm_step['Value']['descr_short'] = 'Save data files to SE and register them in DFC'
        i_step += 1

        # Upload and register log file
        file_meta_data = {}
        file_md_json = json.dumps(file_meta_data)
        log_file_pattern = 'Data/corsika/run*/run*.log'
        scripts = '../CTADIRAC/Core/scripts/'
        log_step = self.setExecutable(scripts + 'cta-prod-managedata.py',
                                      arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log" %
                                      (md_json, md_field_json, file_md_json,
                                       self.base_path, log_file_pattern, self.package,
                                       self.program_category, self.catalogs),
                                      logFile='LogManagement_Log.txt')
        log_step['Value']['name'] = 'Step%s_LogManagement' % i_step
        log_step['Value']['descr_short'] = 'Save log to SE and register them in DFC'
        i_step += 1

        # Step 6 - debug only
        if debug:
            ls_step = self.setExecutable('/bin/ls -alhtr', logFile='LS_End_Log.txt')
            ls_step['Value']['name'] = 'Step%s_LSHOME_End' % i_step
            ls_step['Value']['descr_short'] = 'list files in Home directory'
            i_step += 1

        # Number of showers is passed via an environment variable
        self.setExecutionEnv({'NSHOW': '%s' % self.n_shower})



""" Manage data and meta-data for PROD3
"""

__RCSID__ = "$Id$"

# generic imports
import os
import glob
import json
import tarfile
import re
import collections

# DIRAC imports
import DIRAC
from DIRAC.Resources.Catalog.FileCatalog import FileCatalog
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from DIRAC.DataManagementSystem.Client.DataManager import DataManager
from DIRAC.Core.Utilities import List
from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations
from DIRAC.Core.Utilities.SiteSEMapping import getSEsForSite
from DIRAC.Interfaces.API.Dirac import Dirac


class ProdDataManager(object):
  """ Manage data and meta-data
  """

  def __init__(self, catalogs=['DIRACFileCatalog']):
    """ Constructor

    """
    self.setupCatalogClient(catalogs)
    self.printCatalogConfig(catalogs)
    self.setupDataManagerClient(catalogs)

  def setupCatalogClient(self, catalogs):
    """ Init FileCatalog client
        Ideally we would like to use only FileCatalog but it doesn't work for setMetadata
        because the returned value has a wrong format. So use of FileCatalogClient instead
    """
    self.fc = FileCatalog(catalogs)
    self.fcc = FileCatalogClient()

  def printCatalogConfig(self, catalogs):
    """ Dumps catalog configuration
    """
    for catalog in catalogs:
      res = self.fc._getCatalogConfigDetails(catalog)
      DIRAC.gLogger.info('CatalogConfigDetails:', res)

  def setupDataManagerClient(self, catalogs):
    """ Init DataManager client
    """
    self.dm = DataManager(catalogs)

  def _getSEList(self, SEType='ProductionOutputs', DataType='SimtelProd'):
    """ get from CS the list of available SE for data upload
    """
    opsHelper = Operations()
    optionName = os.path.join(SEType, DataType)
    SEList = opsHelper.getValue(optionName, [])
    SEList = List.randomize(SEList)
    DIRAC.gLogger.notice('List of %s SE: %s ' % (SEType, SEList))

    # # Check if the local SE is in the list. If yes try it first by reversing list order
    localSEList = []
    res = getSEsForSite(DIRAC.siteName())
    if res['OK']:
      localSEList = res['Value']

    retainedlocalSEList = []
    for localSE in localSEList:
      if localSE in SEList:
        DIRAC.gLogger.notice('The local Storage Element is an available SE: ', localSE)
        retainedlocalSEList.append(localSE)
        SEList.remove(localSE)

    SEList = retainedlocalSEList + SEList
    if len(SEList) == 0:
      return DIRAC.S_ERROR('Error in building SEList')

    return DIRAC.S_OK(SEList)

  def _checkemptydir(self, path):
    """ check that the directory is not empty
    """
    if len(glob.glob(path)) == 0:
      error = 'Empty directory: %s' % path
      return DIRAC.S_ERROR(error)
    else:
      return DIRAC.S_OK()

  def _getRunPath(self, filemetadata):
    """ format path to string with 1 digit precision
        run_number is taken from filemetadata
        filemetadata can be a dict or the run_number itself
    """
    if isinstance(filemetadata, dict):
      run_number = int(filemetadata['runNumber'])
    else:
      run_number = int(filemetadata)
    run_numberMod = run_number % 1000
    runpath = '%03dxxx' % ((run_number - run_numberMod) / 1000)
    return runpath

  def _formatPath(self, path):
    """ format path to string with 1 digit precision
    """
    if isinstance(path, float) or isinstance(path, int):
      path = '%.1f' % path
    return str(path)

  def createTarLogFiles(self, inputpath, tarname):
    """ create tar of log and histogram files
    """
    tarmode = 'w:gz'
    tar = tarfile.open(tarname, tarmode)

    for subdir in ['Log/*', 'Histograms/*']:
      logdir = os.path.join(inputpath, subdir)

      res = self._checkemptydir(logdir)
      if not res['OK']:
        return res

      for localfile in glob.glob(logdir):
        tar.add(localfile, arcname=localfile.split('/')[-1])

    tar.close()

    return DIRAC.S_OK()

  def createMDStructure(self, metadata, metadatafield, basepath, program_category):
    """ create meta data structure
    """
    # ## Add metadata fields to the DFC
    mdfield = json.loads(metadatafield)
    for key, value in mdfield.items():
      res = self.fc.addMetadataField(key, value)
      if not res['OK']:
        return res

    # ## Create the directory structure
    md = json.loads(metadata, object_pairs_hook=collections.OrderedDict)

    path = basepath
    process_program = program_category + '_prog'
    for key, value in collections.OrderedDict((k, md[k]) for k in (
            'site', 'particle', process_program) if k in md).items():
      path = os.path.join(path, self._formatPath(value))
      res = self.fc.createDirectory(path)
      if not res['OK']:
        return res

      # Set directory metadata for each subdir: 'site', 'particle', 'process_program'
      res = self.fcc.setMetadata(path, {key: value})
      if not res['OK']:
        return res

    # Create the TransformationID subdir and set MD
    # ## Get the TransformationID
    if 'JOBID' in os.environ:
      jobID = os.environ['JOBID']
      dirac = Dirac()
      res = dirac.getJobJDL(jobID)
      TransformationID = '0000'
      if 'TransformationID' in res['Value']:
        TransformationID = res['Value']['TransformationID']

    path = os.path.join(path, TransformationID)
    res = self.fc.createDirectory(path)
    if not res['OK']:
      return res

    process_program_version = process_program + '_version'
    res = self.fcc.setMetadata(
        path,
        dict(
            (k,
             md[k]) for k in (
                'phiP',
                'thetaP',
                'array_layout',
                process_program_version) if k in md))
    if not res['OK']:
      return res

    # Create the Data and Log subdirs and set MD
    Transformation_path = path
    for subdir in ['Data', 'Log']:
      path = os.path.join(Transformation_path, subdir)
      res = self.fc.createDirectory(path)
      if not res['OK']:
        return res

    # Set metadata if not already defined
      res = self.fcc.getDirectoryUserMetadata(path)
      if not res['OK']:
        return res
      if 'outputType' not in res['Value']:
        res = self.fcc.setMetadata(path, {'outputType': subdir})
        if not res['OK']:
          return res

    # MD for the Data directory - data_level and configuration_id
    path = os.path.join(Transformation_path, 'Data')
    # Set metadata if not already defined
    res = self.fcc.getDirectoryUserMetadata(path)
    if not res['OK']:
      return res
    if 'data_level' or 'configuration_id' or 'merged' not in res['Value']:
      res = self.fcc.setMetadata(path, {'data_level': md['data_level'],
                                        'configuration_id': md['configuration_id'],
                                        'merged': md.get('merged', 0)  # new md, not defined for older tasks
                                        })
      if not res['OK']:
        return res

    return DIRAC.S_OK(Transformation_path)

  def putAndRegister(self, lfn, localfile, filemetadata, DataType='SimtelProd'):
    """ put and register one file and set metadata
    """
    # ## Get the list of Production SE
    res = self._getSEList('ProductionOutputs', DataType)
    if res['OK']:
      ProductionSEList = res['Value']
    else:
      return res

    # ## Get the list of Failover SE
    res = self._getSEList('ProductionOutputsFailover', DataType)
    if res['OK']:
      FailoverSEList = res['Value']
    else:
      return res

    # ## Upload file to a Production SE
    res = self._putAndRegisterToSEList(lfn, localfile, ProductionSEList)
    if not res['OK']:
      DIRAC.gLogger.error('Failed to upload file to any Production SE: %s' % ProductionSEList)
      # ## Upload file to a Failover SE
      res = self._putAndRegisterToSEList(lfn, localfile, FailoverSEList)
      if not res['OK']:
        return DIRAC.S_ERROR('Failed to upload file to any Failover SE: %s' % FailoverSEList)

    # ## Set file metadata: jobID, subarray, sct
    if res['OK']:
      fmd = json.loads(filemetadata)
      if 'JOBID' in os.environ:
        fmd.update({'jobID': os.environ['JOBID']})
      filename = os.path.basename(localfile)
      # set subarray and sct md from filename
      p = re.compile('subarray-\\d+')
      if p.search(filename) is not None:
        subarray = p.search(filename).group()
        fmd.update({'subarray': subarray})
      sct = 'False'
      p = re.compile('nosct')
      psct = re.compile('sct')
      if p.search(filename) is None and psct.search(filename) is not None:
        sct = 'True'
      # ## Set sct flag only for production data
      res = self.fcc.getFileUserMetadata(lfn)
      if DataType == 'SimtelProd' and res['Value']['outputType'] == 'Data':
        fmd.update({'sct': sct})
      res = self.fc.setMetadata(lfn, fmd)
      if not res['OK']:
        return res

      return DIRAC.S_OK()

  def _putAndRegisterToSEList(self, lfn, localfile, SEList):
    """ put and register one file to one SE in the SEList
    """
    # ## Try to upload file to a SE in the list
    for SE in SEList:
      msg = 'Try to upload local file: %s \nwith LFN: %s \nto %s' % (localfile, lfn, SE)
      DIRAC.gLogger.notice(msg)
      res = self.dm.putAndRegister(lfn, localfile, SE)
      DIRAC.gLogger.notice(res)
      # ##  check if failed
      if not res['OK']:
        DIRAC.gLogger.error('Failed to putAndRegister %s \nto %s \nwith message: %s' % (lfn, SE, res['Message']))
        DIRAC.gLogger.notice('Trying to clean up %s' % lfn)
        res = self.dm.removeFile(lfn)
        if res['OK']:
          DIRAC.gLogger.notice(
              'Successfully removed %s \n that was not supposed to have been uploaded successfully' %
              lfn)
        continue
      elif lfn in res['Value']['Failed']:
        DIRAC.gLogger.error('Failed to putAndRegister %s to %s' % (lfn, SE))
        DIRAC.gLogger.notice('Trying to clean up %s' % lfn)
        res = self.dm.removeFile(lfn)
        if res['OK']:
          DIRAC.gLogger.notice(
              'Successfully removed %s \n that was not supposed to have been uploaded successfully' %
              lfn)
        continue
      else:
        return DIRAC.S_OK()
    return DIRAC.S_ERROR()

  def cleanLocalFiles(self, datadir, pattern='*'):
    """ remove files matching pattern in datadir
    """

    for localfile in glob.glob(os.path.join(datadir, pattern)):
      DIRAC.gLogger.notice('Removing local file: ', localfile)
      os.remove(localfile)

    return DIRAC.S_OK()



""" Prod3 MC Pipe Script to create a Transformation
          JB, LA April 2017
"""

__RCSID__ = "$Id$"

from DIRAC.Core.Base import Script
Script.setUsageMessage('\n'.join([__doc__.split('\n')[1],
                                  'Usage:',
                                  '  %s array_layout site particle pointing_dir\
                                   zenith_angle nShower' % Script.scriptName,
                                  'Arguments:',
                                  '  array_layout: demo',
                                  '  site: Paranal or LaPalma',
                                  '  particle: gamma, proton, electron',
                                  '  pointing_dir: North or South',
                                  '  zenith_agle: 20',
                                  '  nShower: from 5 to 25000',
                                  '\ne.g: %s Baseline Paranal gamma South 20 5' % Script.scriptName,
                                  ]))

Script.parseCommandLine()

import DIRAC
from DIRAC import gLogger
from DIRAC.TransformationSystem.Client.Transformation import Transformation
from DIRAC.Core.Workflow.Parameter import Parameter
from CTADIRAC.Interfaces.API.Prod3MCPipeBaselineNSBJob import Prod3MCPipeBaselineNSBJob

def submit_ts(job):
  """ Create a transformation executing the job workflow
  """
  # Temporary fix to initialize JOB_ID #######
  job.workflow.addParameter(Parameter("JOB_ID", "000000", "string", "", "",
                                      True, False, "Temporary fix"))
  job.workflow.addParameter(Parameter("PRODUCTION_ID", "000000", "string",
                                      "", "", True, False, "Temporary fix"))
  job.setType('MCSimulation')  # Used for the JobType plugin

  trans = Transformation()
  # This must be unique. If not set it's asked in the prompt
  # t.setTransformationName("Prod3Exemple")
  trans.setType("MCSimulation")
  trans.setDescription("MC Prod3 BaseLine NSB test")
  trans.setLongDescription("corsika-simtel production")  # mandatory
  trans.setBody(job.workflow.toXML())

  res = trans.addTransformation()  # Transformation is created here

  if not res['OK']:
    gLogger.error(res['Message'])
    DIRAC.exit(-1)

  trans.setStatus("Active")
  trans.setAgentType("Automatic")

  return res

#########################################################


def run_prod3(args=None):
  """ Simple wrapper to create a Prod3MCPipeBaselineJob and setup parameters
  from positional arguments given on the command line.

  Parameters:
  args -- a list of 6 strings corresponding to job arguments
  array_layout site particle pointing_dir zenith_angle nShower
  demo LaPalma  gamma South 20 1000
  """
  # get arguments
  layout = args[0]
  site = args[1]
  particle = args[2]
  pointing = args[3]
  zenith = args[4]
  n_shower = args[5]

  # Main Script ###
  job = Prod3MCPipeBaselineNSBJob()

  # override for testing
  job.setName('BL_NSB_LaPalma_20deg_%s' % particle)

  # package and version
  job.setPackage('corsika_simhessarray')
  job.setVersion('2017-09-01-p1')  # final with fix for gamma-diffuse
  job.configuration_id = 1  # 0 exists already
  job.N_output_files = 2  # La Palma

  # layout, site, particle, pointing direction, zenith angle
  # demo, LaPalma,  gamma, South,  20
  job.setArrayLayout(layout)
  job.setSite(site)
  job.setParticle(particle)
  job.setPointingDir(pointing)
  job.setZenithAngle(zenith)

  # 5 is enough for testing
  job.setNShower(n_shower)

  # Set the startrunNb here (it will be added to the Task_ID)
  start_run_nb = '0'
  job.setStartRunNumber(start_run_nb)

  # set run number for TS submission: JOB_ID variable left for
  # dynamic resolution during the Job. It corresponds to the Task_ID
  job.setRunNumber('@{JOB_ID}')

  # get dirac log files
  job.setOutputSandbox(['*Log.txt'])

  # add the sequence of executables
  job.setupWorkflow(debug=True)

  # submit to the Transformation System
  res = submit_ts(job)

  # debug
  Script.gLogger.info(job.workflow)

  return res


#########################################################
if __name__ == '__main__':
  """ Do things
  """
  ARGS = Script.getPositionalArgs()
  if len(ARGS) != 6:
    Script.showHelp()
  try:
    RES = run_prod3(ARGS)
    if not RES['OK']:
      DIRAC.gLogger.error(RES['Message'])
      DIRAC.exit(-1)
    else:
      DIRAC.gLogger.notice('Done')
  except RuntimeError:
    DIRAC.gLogger.exception()
    DIRAC.exit(-1)



#!/usr/bin/env python
""" Data management script for production
    create DFC MetaData structure put and register files in DFC
    should work for corsika, simtel and EventDisplay output
    derived from cta-prod3-managedata.py and cta-analysis-managedata.py
                    JB September 2018
"""

__RCSID__ = "$Id$"

# generic imports
import os
import glob
import json

# DIRAC imports
import DIRAC
# To do: Change import
from DIRAC.Core.Base import Script
Script.parseCommandLine()

# Specific DIRAC imports
from CTADIRAC.Core.Utilities.tool_box import run_number_from_filename
from CTADIRAC.Core.Workflow.Modules.ProdDataManager import ProdDataManager

def main():
  """ simple wrapper to put and register all production files

  Keyword arguments:
  args -- a list of arguments in order []
  """
  args = Script.getPositionalArgs()
  metadata = args[0]
  metadata_fields = args[1]
  file_metadata = args[2]
  base_path = args[3]
  output_pattern = args[4]
  package = args[5]
  program_category = args[6]
  catalogs = args[7]
  output_type = args[8]

  # Load catalogs
  catalogs_json = json.loads(catalogs)

  # Create MD structure
  prod3dm = ProdDataManager(catalogs_json)
  result = prod3dm.createMDStructure(metadata, metadata_fields, base_path, program_category)
  if result['OK']:
    path = result['Value']
  else:
    return result

  # Check the content of the output directory
  result = prod3dm._checkemptydir(output_pattern)
  if not result['OK']:
    return result

  # Dump the list of output LFNs
  file = open("output_lfns.txt", 'w')

  # Loop over each file and upload and register
  for localfile in glob.glob(output_pattern):
    file_name = os.path.basename(localfile)
    # Check run number, assign one as file metadata if needed
    fmd_dict = json.loads(file_metadata)
    try:
      run_number = run_number_from_filename(file_name, package)
    except BaseException:
      run_number = -9999
      DIRAC.gLogger.notice('Could not get a correct run number, assigning -9999')
    fmd_dict['runNumber'] = '%08d' % int(run_number)
    # get the output file path
    run_path = prod3dm._getRunPath(fmd_dict)
    lfn = os.path.join(path, output_type, run_path, file_name)
    fmd_json = json.dumps(fmd_dict)
    result = prod3dm.putAndRegister(lfn, localfile, fmd_json, package)
    if not result['OK']:
      return result
    file.write(lfn)
    file.write('\n')
  file.close()

  DIRAC.exit()

####################################################
if __name__ == '__main__':
  main()